from jgtconstants import (MFI_FADE, MFI_FADE_ID, MFI_FADE_STR, MFI_FAKE,
                          MFI_FAKE_ID, MFI_FAKE_STR, MFI_GREEN, MFI_GREEN_ID,
                          MFI_GREEN_STR, MFI_SQUAT, MFI_SQUAT_ID,
                          MFI_SQUAT_STR)

from jgtutils.jgtconstants import MFI_SIGNAL as MFI_DEFAULT_COLNAME
from jgtutils.jgtconstants import ZONE_SIGNAL as ZONE_DEFAULT_COLNAME


def mfi_str_to_id(mfi_str:str)->int:
    if mfi_str == MFI_SQUAT_STR:
        return MFI_SQUAT_ID
    elif mfi_str == MFI_FAKE_STR:
        return MFI_FAKE_ID
    elif mfi_str == MFI_FADE_STR:
        return MFI_FADE_ID
    elif mfi_str == MFI_GREEN_STR:
        return MFI_GREEN_ID
    else:
        return 0

def mfi_signal_to_str(mfi_signal:int)->str:
    if mfi_signal == MFI_SQUAT:
        return MFI_SQUAT_STR
    elif mfi_signal == MFI_FAKE:
        return MFI_FAKE_STR
    elif mfi_signal == MFI_FADE:
        return MFI_FADE_STR
    elif mfi_signal == MFI_GREEN:
        return MFI_GREEN_STR
    else:
        return "0"

def get_mfi_features_column_list_by_timeframe(t:str,mfi_colname=""):
    """
    Get the list of columns that are MFI features for the given timeframe and its related timeframes.
    
    Parameters:
    t (str): The timeframe to get the MFI features columns for.
    mfi_colname (str): The name of the MFI column to use. Default is MFI_VAL (plan to upgrade to MFI_SIGNAL)
    
    Returns:
    list: The list of columns that are MFI features for the given timeframe and its related timeframes
    """
    if mfi_colname=="":
        mfi_colname=MFI_DEFAULT_COLNAME
    mfi_str_selected_columns = [mfi_colname+'_M1',mfi_colname+'_W1']
    
    if t=='H4' or t=='H8' or t=='H6' or t=='H1' or t=='m15' or t=='m5':
      mfi_str_selected_columns.append(mfi_colname+'_D1')
      
    if t=='H1' or t=='m15' or t=='m5':
        mfi_str_selected_columns.append(mfi_colname+'_H4')
        
    if t=='m15' or t=='m5':
        mfi_str_selected_columns.append(mfi_colname+'_H1')
    
    if t=='m5':
        mfi_str_selected_columns.append(mfi_colname+'_m15')
        
    mfi_str_selected_columns.append(mfi_colname)
    return mfi_str_selected_columns



from jgtconstants import (ZONE_BUY_ID, ZONE_BUY_STR, ZONE_INT, ZONE_NEUTRAL_ID,
                          ZONE_NEUTRAL_STR, ZONE_SELL_ID, ZONE_SELL_STR)


def zone_str_to_id(zone_str:str)->int:
    if zone_str == ZONE_BUY_STR:
        return ZONE_BUY_ID
    elif zone_str == ZONE_SELL_STR:
        return ZONE_SELL_ID
    elif zone_str == ZONE_NEUTRAL_STR:
        return ZONE_NEUTRAL_ID
    else:
        return ZONE_NEUTRAL_ID

def zone_id_to_str(zone_id:int)->str:
    if zone_id == ZONE_BUY_ID:
        return ZONE_BUY_STR
    elif zone_id == ZONE_SELL_ID:
        return ZONE_SELL_STR
    elif zone_id == ZONE_NEUTRAL_ID:
        return ZONE_NEUTRAL_STR
    else:
        return ZONE_NEUTRAL_STR




#get_zone_columns_list
#get_mfi_features_column_list_by_timeframe
def get_zone_features_column_list_by_timeframe(t:str,zone_colname=""):
    """
    Get the list of columns that are ZONE features for the given timeframe and its related timeframes.
    
    Parameters:
    t (str): The timeframe to get the list of ZONE features for.
    zone_colname (str): The name of the ZONE column to use. If not provided, the default ZCOL is used. (Planning to use ZONE_SIGNAL)
    
    Returns:
    list: The list of columns that are ZONE features for the given timeframe and its related timeframes.
    
    """
    if zone_colname=="":
        zone_colname=ZONE_DEFAULT_COLNAME
    
    zcol_ctx_selected_columns = [zone_colname+'_M1',zone_colname+'_W1']
    
    if t=='H4' or t=='H8' or t=='H6' or t=='H1' or t=='m15' or t=='m5':
      zcol_ctx_selected_columns.append(zone_colname+'_D1')
      
    if t=='H1' or t=='m15' or t=='m5':
        zcol_ctx_selected_columns.append(zone_colname+'_H4')
        
    if t=='m15' or t=='m5':
        zcol_ctx_selected_columns.append(zone_colname+'_H1')
    
    if t=='m5':
        zcol_ctx_selected_columns.append(zone_colname+'_m15')
        
    zcol_ctx_selected_columns.append(zone_colname)
    return zcol_ctx_selected_columns
  


#!/usr/bin/env python

import argparse

from jgtpov import calculate_tlid_range as get_tlid_range


def main():
    # Create the parser
    parser = argparse.ArgumentParser(description="Calculate tlid range")

    # Add the arguments
    parser.add_argument("-e", type=str, help="The end datetime")
    parser.add_argument("-t", type=str, help="The timeframe")
    parser.add_argument("-c", type=int, help="The number of periods")

    # Parse the arguments
    args = parser.parse_args()

    # Call get_tlid_range with the arguments
    result = get_tlid_range(args.e, args.t, args.c)

    # Print the result
    print(result)


if __name__ == "__main__":
    main()



from jgtconstants import *


def get_constant_col_type_by_name(colname:str):
    """
    Get the type of the column based on the column name.
    
    Parameters:
    colname (str): The column name to get the type for.
    
    Returns:
    str: The type of the column based on the column name.
    
    """
    
    """
    AOAZ_TYPE=int
AOBZ_TYPE=int
ZLC_TYPE=int
MFI_SIGNAL_TYPE = int
MFI_SQUAT_TYPE = int
MFI_GREEN_TYPE = int
MFI_FADE_TYPE = int
MFI_FAKE_TYPE = int
ZLCB_TYPE=int
ZLCS_TYPE=int
ZONE_SIGNAL_TYPE= int
SZ_TYPE=int
BZ_TYPE=int
SS_TYPE=int
SB_TYPE=int
PRICE_PEAK_ABOVE_TYPE = int
AO_PEAK_ABOVE_TYPE = int
PRICE_PEAK_BELLOW_TYPE = int
AO_PEAK_BELLOW_TYPE = int
FDB_TARGET_TYPE = float
VECTOR_AO_FDB_COUNT_TYPE = int
VECTOR_AO_FDBS_COUNT_TYPE = int
VECTOR_AO_FDBB_COUNT_TYPE= int
    """
    if colname == AOAZ:
        return AOAZ_TYPE
    elif colname == AOBZ:
        return AOBZ_TYPE
    elif colname == ZLC:
        return ZLC_TYPE
    elif colname == MFI_SIGNAL:
        return MFI_SIGNAL_TYPE
    elif colname == MFI_SQUAT:
        return MFI_SQUAT_TYPE
    elif colname == MFI_GREEN:
        return MFI_GREEN_TYPE
    elif colname == MFI_FADE:
        return MFI_FADE_TYPE
    elif colname == MFI_FAKE:
        return MFI_FAKE_TYPE
    elif colname == ZLCB:
        return ZLCB_TYPE
    elif colname == ZLCS:
        return ZLCS_TYPE
    elif colname == ZONE_SIGNAL:
        return ZONE_SIGNAL_TYPE
    elif colname == SZ:
        return SZ_TYPE
    elif colname == BZ:
        return BZ_TYPE
    elif colname == SS:
        return SS_TYPE
    elif colname == SB:
        return SB_TYPE
    elif colname == PRICE_PEAK_ABOVE:
        return PRICE_PEAK_ABOVE_TYPE
    elif colname == AO_PEAK_ABOVE:
        return AO_PEAK_ABOVE_TYPE
    elif colname == PRICE_PEAK_BELLOW:
        return PRICE_PEAK_BELLOW_TYPE
    elif colname == AO_PEAK_BELLOW:
        return AO_PEAK_BELLOW_TYPE
    elif colname == FDB_TARGET:
        return FDB_TARGET_TYPE
    elif colname == VECTOR_AO_FDB_COUNT:
        return VECTOR_AO_FDB_COUNT_TYPE
    elif colname == VECTOR_AO_FDBS_COUNT:
        return VECTOR_AO_FDBS_COUNT_TYPE
    elif colname == VECTOR_AO_FDBB_COUNT:
        return VECTOR_AO_FDBB_COUNT_TYPE
    elif colname == ACB:
        return ACB_TYPE
    elif colname == ACS:
        return ACS_TYPE
    else:
        return None
      


def get_dtype_definitions():
    """
    Get the dictionary of column names and their types.
    
    Returns:
    dict: The dictionary of column names and their types.
    
    """
    dtype_definitions = {
        AOAZ: AOAZ_TYPE,
        AOBZ: AOBZ_TYPE,
        ZLC: ZLC_TYPE,
        MFI_SIGNAL: MFI_SIGNAL_TYPE,
        MFI_SQUAT: MFI_SQUAT_TYPE,
        MFI_GREEN: MFI_GREEN_TYPE,
        MFI_FADE: MFI_FADE_TYPE,
        MFI_FAKE: MFI_FAKE_TYPE,
        ZLCB: ZLCB_TYPE,
        ZLCS: ZLCS_TYPE,
        ZONE_SIGNAL: ZONE_SIGNAL_TYPE,
        SZ: SZ_TYPE,
        BZ: BZ_TYPE,
        SS: SS_TYPE,
        SB: SB_TYPE,
        PRICE_PEAK_ABOVE: PRICE_PEAK_ABOVE_TYPE,
        AO_PEAK_ABOVE: AO_PEAK_ABOVE_TYPE,
        PRICE_PEAK_BELLOW: PRICE_PEAK_BELLOW_TYPE,
        AO_PEAK_BELLOW: AO_PEAK_BELLOW_TYPE,
        FDB_TARGET: FDB_TARGET_TYPE,
        VECTOR_AO_FDB_COUNT: VECTOR_AO_FDB_COUNT_TYPE,
        VECTOR_AO_FDBS_COUNT: VECTOR_AO_FDBS_COUNT_TYPE,
        VECTOR_AO_FDBB_COUNT: VECTOR_AO_FDBB_COUNT_TYPE,
        ACB: ACB_TYPE,ACS: ACS_TYPE
    }
    return dtype_definitions
  
DTYPE_DEFINITIONS=get_dtype_definitions()


import unittest
from unittest.mock import patch

# Assuming read_fx_str_from_config is defined in jgtcommon.py
from jgtcommon import read_fx_str_from_config


class TestReadFxStrFromConfig(unittest.TestCase):
    @patch('jgtcommon.readconfig')
    def test_read_fx_str_from_config_demo_false(self, mock_readconfig):
        # Mock the config data for demo=False
        mock_readconfig.return_value = {
            "user_id": "010101010",
            "account": "0001",
            "password": "MyRealFXCMPassword",
            "url": "https://www.fxcorporate.com/Hosts.jsp",
            "connection": "Real"
        }
        
        expected_result = (
            "010101010",
            "MyRealFXCMPassword",
            "https://www.fxcorporate.com/Hosts.jsp",
            "Real",
            "0001"
        )
        
        result = read_fx_str_from_config(demo=False)
        self.assertEqual(result, expected_result)

    @patch('jgtcommon.readconfig')
    def test_read_fx_str_from_config_demo_true(self, mock_readconfig):
        # Mock the config data for demo=True
        mock_readconfig.return_value = {
            "user_id": "U10D8798797",
            "password": "MyDemoFXCMPassword",
            "account": "002342343",
            "url": "https://www.fxcorporate.com/Hosts.jsp",
            "connection": "Demo"
        }
        
        expected_result = (
            "U10D8798797",
            "MyDemoFXCMPassword",
            "https://www.fxcorporate.com/Hosts.jsp",
            "Demo",
            "002342343"
        )
        
        result = read_fx_str_from_config(demo=True)
        self.assertEqual(result, expected_result)

if __name__ == '__main__':
    unittest.main()


import os

import sys

sys.path.insert(0, os.path.abspath(os.path.dirname(__file__)))

import jgtcommon

def parse_args():
    parser = jgtcommon.new_parser("Timeframe Test settings","Test loading custom timeframes from settings",enable_specified_settings=True)
    #parser=jgtcommon.add_settings_argument(parser)
    #parser=jgtcommon._preload_settings_from_args(parser)
    
    parser= jgtcommon.add_instrument_timeframe_arguments(parser)
    args = jgtcommon.parse_args(parser)
    return args

def main():
    args = parse_args()
    #settings = jgtcommon.load_settings(args)
    print("Settings loaded:")
    print(jgtcommon.settings)
    
if __name__ == '__main__':
    main()
    



import json
import os
import unittest
from unittest.mock import patch

# Assuming read_fx_str_from_config is defined in jgtcommon.py
from jgtcommon import read_fx_str_from_config

# Path to the sample config JSON file
TEST_CONFIG_JSON_SAMPLE_PATH = os.path.join(os.path.dirname(__file__), 'test_config_sample.json')

class TestReadFxStrFromConfigSampleFile(unittest.TestCase):
    @patch('jgtcommon.readconfig')
    def test_read_fx_str_from_configfile_demo_false(self, mock_readconfig):
        # Load the config data from the sample JSON file
        with open(TEST_CONFIG_JSON_SAMPLE_PATH, 'r') as file:
            config_data = json.load(file)
        
        # Mock the config data for demo=False
        mock_readconfig.return_value = config_data
        
        expected_result = (
            "010101010",
            "MyRealFXCMPassword",
            "https://www.fxcorporate.com/Hosts.jsp",
            "Real",
            "0001"
        )
        
        
        result = read_fx_str_from_config(demo=False)
        self.assertEqual(result, expected_result)

    @patch('jgtcommon.readconfig')
    def test_read_fx_str_from_configfile_demo_true(self, mock_readconfig):
        # Load the config data from the sample JSON file
        with open(TEST_CONFIG_JSON_SAMPLE_PATH, 'r') as file:
            config_data = json.load(file)
        
        # Mock the config data for demo=True
        mock_readconfig.return_value = config_data
        
        expected_result = (
            "U10D8798797",
            "MyDemoFXCMPassword",
            "https://www.fxcorporate.com/Hosts.jsp",
            "Demo",
            "002342343"
        )
        
        result = read_fx_str_from_config(demo=True)
        self.assertEqual(result, expected_result)

if __name__ == '__main__':
    unittest.main()



import datetime
import json
import os
import sys

sys.path.insert(0, os.path.abspath(os.path.dirname(__file__)))

from jgtfxhelper import offer_id_to_instrument


class FXTrades:
    def __init__(self, trades=None):
        self.trades = trades or []
    
    def add_trade(self, trade_data):
        #support json string or string
        input_is_string = isinstance(trade_data, str)
        if input_is_string:
            try:
                trade_data = FXTrade.from_string(trade_data)
            except ValueError:
                trade_data = FXTrade.from_json_string(trade_data)
        else:
            self.trades.append(trade_data)
    
    def to_dict(self):
        return {
            "trades": [trade.to_dict() for trade in self.trades]
        }
    def tojson(self, indent=2):
        return json.dumps(self.to_dict(), indent=indent)
    
    @classmethod
    def from_string(cls, trades_string):
        trades = []
        for trade_string in trades_string.split('\n'):
            trades.append(FXTrade.from_string(trade_string))
        return cls(trades)

class FXTrade:
    def __init__(self, trade_id, account_id, account_name, account_kind, offer_id, amount, buy_sell, open_rate, open_time, open_quote_id, open_order_id, open_order_req_id, open_order_request_txt, commission, rollover_interest, trade_id_origin, used_margin, value_date, parties, dividends, pl, gross_pl, close, stop, limit, stop_order_id, limit_order_id, instrument, trail_rate, trail_step, close_commission):
        self.trade_id = trade_id
        self.account_id = account_id
        self.account_name = account_name
        self.account_kind = account_kind
        self.offer_id = offer_id
        self.amount = amount
        self.buy_sell = buy_sell
        self.open_rate = open_rate
        self.open_time = datetime.datetime.strptime(open_time, '%Y-%m-%d %H:%M:%S') if open_time else None
        self.open_quote_id = open_quote_id
        self.open_order_id = open_order_id
        self.open_order_req_id = open_order_req_id
        self.open_order_request_txt = open_order_request_txt
        self.commission = commission
        self.rollover_interest = rollover_interest
        self.trade_id_origin = trade_id_origin
        self.used_margin = used_margin
        self.value_date = value_date
        self.parties = parties
        self.dividends = dividends
        self.pl = pl
        self.gross_pl = gross_pl
        self.close = close
        self.stop = stop
        self.limit = limit
        self.stop_order_id = stop_order_id
        self.limit_order_id = limit_order_id
        self.instrument = instrument
        self.trail_rate = trail_rate
        self.trail_step = trail_step
        self.close_commission = close_commission

    def to_dict(self):
        return {
            "trade_id": self.trade_id,
            "account_id": self.account_id,
            "account_name": self.account_name,
            "account_kind": self.account_kind,
            "offer_id": self.offer_id,
            "amount": self.amount,
            "buy_sell": self.buy_sell,
            "open_rate": self.open_rate,
            "open_time": str(self.open_time),
            "open_quote_id": self.open_quote_id,
            "open_order_id": self.open_order_id,
            "open_order_req_id": self.open_order_req_id,
            "open_order_request_txt": self.open_order_request_txt,
            "commission": self.commission,
            "rollover_interest": self.rollover_interest,
            "trade_id_origin": self.trade_id_origin,
            "used_margin": self.used_margin,
            "value_date": self.value_date,
            "parties": self.parties,
            "dividends": self.dividends,
            "pl": self.pl,
            "gross_pl": self.gross_pl,
            "close": self.close,
            "stop": self.stop,
            "limit": self.limit,
            "stop_order_id": self.stop_order_id,
            "limit_order_id": self.limit_order_id,
            "instrument": self.instrument,
            "trail_rate": self.trail_rate,
            "trail_step": self.trail_step,
            "close_commission": self.close_commission
        }
    
    def from_json_string(self, json_string):
        trade_data = json.loads(json_string)
        return FXTrade(
            trade_id=trade_data.get('trade_id', 0),
            account_id=trade_data.get('account_id', 0),
            account_name=trade_data.get('account_name', ''),
            account_kind=trade_data.get('account_kind', 0),
            offer_id=trade_data.get('offer_id', 0),
            amount=trade_data.get('amount', 0),
            buy_sell=trade_data.get('buy_sell', ''),
            open_rate=trade_data.get('open_rate', 0.0),
            open_time=trade_data.get('open_time', ''),
            open_quote_id=trade_data.get('open_quote_id', ''),
            open_order_id=trade_data.get('open_order_id', ''),
            open_order_req_id=trade_data.get('open_order_req_id', ''),
            open_order_request_txt=trade_data.get('open_order_request_txt', ''),
            commission=trade_data.get('commission', 0.0),
            rollover_interest=trade_data.get('rollover_interest', 0.0),
            trade_id_origin=trade_data.get('trade_id_origin', ''),
            used_margin=trade_data.get('used_margin', 0.0),
            value_date=trade_data.get('value_date', ''),
            parties=trade_data.get('parties', ''),
            dividends=trade_data.get('dividends', 0.0),
            pl=trade_data.get('pl', 0.0),
            gross_pl=trade_data.get('gross_pl', 0.0),
            close=trade_data.get('close', 0.0),
            stop=trade_data.get('stop', 0.0),
            limit=trade_data.get('limit', 0.0),
            stop_order_id=trade_data.get('stop_order_id', ''),
            limit_order_id=trade_data.get('limit_order_id', ''),
            instrument=trade_data.get('instrument', ''),
            trail_rate=trade_data.get('trail_rate', 0.0),
            trail_step=trade_data.get('trail_step', 0.0),
            close_commission=trade_data.get('close_commission', 0.0)
        )
    
    def tojson(self,indent=2):
        return json.dumps(self.to_dict(), indent=indent)
    
    @classmethod
    def from_string(cls, trade_string):
        trade_data = {}
        trades = trade_string.split(';')
        #raise if not a valid trade string
        if len(trades) < 2:
            raise ValueError("Invalid trade string")
        for item in trades:
            if '=' in item:
                key, value = item.split('=')
                key = key.strip()
                value = value.strip()
                trade_data[key] = value
        return cls(
            trade_id=int(trade_data.get('trade_id', 0)),
            account_id=int(trade_data.get('account_id', 0)),
            account_name=trade_data.get('account_name', ''),
            account_kind=int(trade_data.get('account_kind', 0)),
            offer_id=int(trade_data.get('offer_id', 0)),
            amount=int(trade_data.get('amount', 0)),
            buy_sell=trade_data.get('buy_sell', ''),
            open_rate=float(trade_data.get('open_rate', 0.0)),
            open_time=trade_data.get('open_time', ''),
            open_quote_id=trade_data.get('open_quote_id', ''),
            open_order_id=trade_data.get('open_order_id', ''),
            open_order_req_id=trade_data.get('open_order_req_id', ''),
            open_order_request_txt=trade_data.get('open_order_request_txt', ''),
            commission=float(trade_data.get('commission', 0.0)),
            rollover_interest=float(trade_data.get('rollover_interest', 0.0)),
            trade_id_origin=trade_data.get('trade_id_origin', ''),
            used_margin=float(trade_data.get('used_margin', 0.0)),
            value_date=trade_data.get('value_date', ''),
            parties=trade_data.get('parties', ''),
            dividends=float(trade_data.get('dividends', 0.0)),
            pl=float(trade_data.get('pl', 0.0)),
            gross_pl=float(trade_data.get('gross_pl', 0.0)),
            close=float(trade_data.get('close', 0.0)),
            stop=float(trade_data.get('stop', 0.0)),
            limit=float(trade_data.get('limit', 0.0)),
            stop_order_id=trade_data.get('stop_order_id', ''),
            limit_order_id=trade_data.get('limit_order_id', ''),
            instrument=trade_data.get('instrument', ''),
            trail_rate=float(trade_data.get('trail_rate', 0.0)),
            trail_step=float(trade_data.get('trail_step', 0.0)),
            close_commission=float(trade_data.get('close_commission', 0.0))
        )

    def __repr__(self):
        return f"Trade(trade_id={self.trade_id}, account_id={self.account_id}, account_name='{self.account_name}', account_kind={self.account_kind}, offer_id={self.offer_id}, amount={self.amount}, buy_sell='{self.buy_sell}', open_rate={self.open_rate}, open_time='{self.open_time}', open_quote_id='{self.open_quote_id}', open_order_id='{self.open_order_id}', open_order_req_id='{self.open_order_req_id}', open_order_request_txt='{self.open_order_request_txt}', commission={self.commission}, rollover_interest={self.rollover_interest}, trade_id_origin='{self.trade_id_origin}', used_margin={self.used_margin}, value_date='{self.value_date}', parties='{self.parties}', dividends={self.dividends}, pl={self.pl}, gross_pl={self.gross_pl}, close={self.close}, stop={self.stop}, limit={self.limit}, stop_order_id='{self.stop_order_id}', limit_order_id='{self.limit_order_id}', instrument='{self.instrument}', trail_rate={self.trail_rate}, trail_step={self.trail_step}, close_commission={self.close_commission})"


class FXOrders:
    def __init__(self, orders=None):
        self.orders = orders or []
    
    def add_order(self, order_data):
        #support json string or string
        input_is_string = isinstance(order_data, str)
        if input_is_string:
            try:
                order_data = FXOrder.from_string(order_data)
            except ValueError:
                order_data = FXOrder.from_json_string(order_data)
        else:
            self.orders.append(order_data)
    
    def to_dict(self):
        return {
            "orders": [order.to_dict() for order in self.orders]
        }
    def tojson(self, indent=2):
        return json.dumps(self.to_dict(), indent=indent)
    
    @classmethod
    def from_string(cls, orders_string):
        orders = []
        for order_string in orders_string.split('\n'):
            orders.append(FXOrder.from_string(order_string))
        return cls(orders)


class FXOrder:
    def __init__(self, order_id, request_id, rate, execution_rate, rate_min, rate_max, trade_id, account_id, account_name, offer_id, net_quantity, buy_sell, stage, type, status, status_time, amount, lifetime, at_market, trail_step, trail_rate, time_in_force, account_kind, request_txt, contingent_order_id, contingency_type, primary_id, origin_amount, filled_amount, working_indicator, peg_type, peg_offset, peg_offset_min, peg_offset_max, expire_date, value_date, parties, side, stop, limit, stop_order_id, limit_order_id, type_stop, type_limit, stop_trail_step, stop_trail_rate):
        self.order_id = order_id
        self.request_id = request_id
        self.rate = rate
        self.execution_rate = execution_rate
        self.rate_min = rate_min
        self.rate_max = rate_max
        self.trade_id = trade_id
        self.account_id = account_id
        self.account_name = account_name
        self.offer_id = offer_id
        self.net_quantity = net_quantity
        self.buy_sell = buy_sell
        self.stage = stage
        self.type = type
        self.status = status
        self.status_time = datetime.datetime.strptime(status_time, '%Y-%m-%d %H:%M:%S')
        self.amount = amount
        self.lifetime = lifetime
        self.at_market = at_market
        self.trail_step = trail_step
        self.trail_rate = trail_rate
        self.time_in_force = time_in_force
        self.account_kind = account_kind
        self.request_txt = request_txt
        self.contingent_order_id = contingent_order_id
        self.contingency_type = contingency_type
        self.primary_id = primary_id
        self.origin_amount = origin_amount
        self.filled_amount = filled_amount
        self.working_indicator = working_indicator
        self.peg_type = peg_type
        self.peg_offset = peg_offset
        self.peg_offset_min = peg_offset_min
        self.peg_offset_max = peg_offset_max
        self.expire_date = datetime.datetime.strptime(expire_date, '%Y-%m-%d %H:%M:%S')
        self.value_date = value_date
        self.parties = parties
        self.side = side
        self.stop = stop
        self.limit = limit
        self.stop_order_id = stop_order_id
        self.limit_order_id = limit_order_id
        self.type_stop = type_stop
        self.type_limit = type_limit
        self.stop_trail_step = stop_trail_step
        self.stop_trail_rate = stop_trail_rate
        self.instrument = offer_id_to_instrument(offer_id)

    def to_dict(self):
        return {
            "order_id": self.order_id,
            "request_id": self.request_id,
            "rate": self.rate,
            "execution_rate": self.execution_rate,
            "rate_min": self.rate_min,
            "rate_max": self.rate_max,
            "trade_id": self.trade_id,
            "account_id": self.account_id,
            "account_name": self.account_name,
            "offer_id": self.offer_id,
            "net_quantity": self.net_quantity,
            "buy_sell": self.buy_sell,
            "stage": self.stage,
            "type": self.type,
            "status": self.status,
            "status_time": str(self.status_time),
            "amount": self.amount,
            "lifetime": self.lifetime,
            "at_market": self.at_market,
            "trail_step": self.trail_step,
            "trail_rate": self.trail_rate,
            "time_in_force": self.time_in_force,
            "account_kind": self.account_kind,
            "request_txt": self.request_txt,
            "contingent_order_id": self.contingent_order_id,
            "contingency_type": self.contingency_type,
            "primary_id": self.primary_id,
            "origin_amount": self.origin_amount,
            "filled_amount": self.filled_amount,
            "working_indicator": self.working_indicator,
            "peg_type": self.peg_type,
            "peg_offset": self.peg_offset,
            "peg_offset_min": self.peg_offset_min,
            "peg_offset_max": self.peg_offset_max,
            "expire_date": str(self.expire_date),
            "value_date": self.value_date,
            "side": self.side,
            "stop": self.stop,
            "limit": self.limit,
            "stop_order_id": self.stop_order_id,
            "limit_order_id": self.limit_order_id,
            "type_stop": self.type_stop,
            "type_limit": self.type_limit,
            "stop_trail_step": self.stop_trail_step,
            "stop_trail_rate": self.stop_trail_rate,
            "instrument": self.instrument
        }

    def tojson(self, indent=2):
        return json.dumps(self.to_dict(), indent=indent)
    
    def from_json_string(self,json_string):
        order_data = json.loads(json_string)
        offer_id = order_data.get('offer_id', 0)
        return FXOrder(
            order_id=order_data.get('order_id', 0),
            request_id=order_data.get('request_id', ''),
            rate=order_data.get('rate', 0.0),
            execution_rate=order_data.get('execution_rate', 0.0),
            rate_min=order_data.get('rate_min', 0.0),
            rate_max=order_data.get('rate_max', 0.0),
            trade_id=order_data.get('trade_id', 0),
            account_id=order_data.get('account_id', 0),
            account_name=order_data.get('account_name', ''),
            offer_id=offer_id,
            net_quantity=order_data.get('net_quantity', False),
            buy_sell=order_data.get('buy_sell', ''),
            stage=order_data.get('stage', ''),
            type=order_data.get('type', ''),
            status=order_data.get('status', ''),
            status_time=order_data.get('status_time', ''),
            amount=order_data.get('amount', 0),
            lifetime=order_data.get('lifetime', 0.0),
            at_market=order_data.get('at_market', 0.0),
            trail_step=order_data.get('trail_step', 0),
            trail_rate=order_data.get('trail_rate', 0.0),
            time_in_force=order_data.get('time_in_force', ''),
            account_kind=order_data.get('account_kind', 0),
            request_txt=order_data.get('request_txt', ''),
            contingent_order_id=order_data.get('contingent_order_id', 0),
            contingency_type=order_data.get('contingency_type', 0),
            primary_id=order_data.get('primary_id', ''),
            origin_amount=order_data.get('origin_amount', 0),
            filled_amount=order_data.get('filled_amount', 0),
            working_indicator=order_data.get('working_indicator', False),
            peg_type=order_data.get('peg_type', ''),
            peg_offset=order_data.get('peg_offset', 0.0),
            peg_offset_min=order_data.get('peg_offset_min', 0.0),
            peg_offset_max=order_data.get('peg_offset_max', 0.0),
            expire_date=order_data.get('expire_date', ''),
            value_date=order_data.get('value_date', ''),
            parties=order_data.get('parties', ''),
            side=order_data.get('side', 0),
            stop=order_data.get('stop', 0.0),
            limit=order_data.get('limit', 0.0),
            stop_order_id=order_data.get('stop_order_id', ''),
            limit_order_id=order_data.get('limit_order_id', ''),
            type_stop=order_data.get('type_stop', 0),
            type_limit=order_data.get('type_limit', 0),
            stop_trail_step=order_data.get('stop_trail_step', 0),
            stop_trail_rate=order_data.get('stop_trail_rate', 0.0),
            instrument=offer_id_to_instrument(offer_id)
        )
    
    @classmethod
    def from_string(cls, order_string):
        order_data = {}
        orders = order_string.split(';')
        #raise if not a valid order string
        if len(orders) < 2:
            raise ValueError("Invalid order string")
        for item in orders:
            if '=' in item:
                key, value = item.split('=')
                key = key.strip()
                value = value.strip()
                order_data[key] = value
        contingent_order_id = order_data.get('contingent_order_id', 0)
        if contingent_order_id == '':
            contingent_order_id='0'
        return cls(
            order_id=int(order_data.get('order_id', 0)),
            request_id=order_data.get('request_id', ''),
            rate=float(order_data.get('rate', 0.0)),
            execution_rate=float(order_data.get('execution_rate', 0.0)),
            rate_min=float(order_data.get('rate_min', 0.0)),
            rate_max=float(order_data.get('rate_max', 0.0)),
            trade_id=int(order_data.get('trade_id', 0)),
            account_id=int(order_data.get('account_id', 0)),
            account_name=order_data.get('account_name', ''),
            offer_id=int(order_data.get('offer_id', 0)),
            net_quantity=order_data.get('net_quantity', 'False') == 'True',
            buy_sell=order_data.get('buy_sell', ''),
            stage=order_data.get('stage', ''),
            type=order_data.get('type', ''),
            status=order_data.get('status', ''),
            status_time=order_data.get('status_time', ''),
            amount=int(order_data.get('amount', 0)),
            lifetime=float(order_data.get('lifetime', 0.0)),
            at_market=float(order_data.get('at_market', 0.0)),
            trail_step=int(order_data.get('trail_step', 0)),
            trail_rate=float(order_data.get('trail_rate', 0.0)),
            time_in_force=order_data.get('time_in_force', ''),
            account_kind=int(order_data.get('account_kind', 0)),
            request_txt=order_data.get('request_txt', ''),
            contingent_order_id=int(contingent_order_id),
            contingency_type=int(order_data.get('contingency_type', 0)),
            primary_id=order_data.get('primary_id', ''),
            origin_amount=int(order_data.get('origin_amount', 0)),
            filled_amount=int(order_data.get('filled_amount', 0)),
            working_indicator=order_data.get('working_indicator', 'False') == 'True',
            peg_type=order_data.get('peg_type', ''),
            peg_offset=float(order_data.get('peg_offset', 0.0)),
            peg_offset_min=float(order_data.get('peg_offset_min', 0.0)),
            peg_offset_max=float(order_data.get('peg_offset_max', 0.0)),
            expire_date=order_data.get('expire_date', ''),
            value_date=order_data.get('value_date', ''),
            parties=order_data.get('parties', ''),
            side=int(order_data.get('side', 0)),
            stop=float(order_data.get('stop', 0.0)),
            limit=float(order_data.get('limit', 0.0)),
            stop_order_id=order_data.get('stop_order_id', ''),
            limit_order_id=order_data.get('limit_order_id', ''),
            type_stop=int(order_data.get('type_stop', 0)),
            type_limit=int(order_data.get('type_limit', 0)),
            stop_trail_step=int(order_data.get('stop_trail_step', 0)),
            stop_trail_rate=float(order_data.get('stop_trail_rate', 0.0))
        )

    def __repr__(self):
        return f"FXOrder(order_id={self.order_id}, request_id='{self.request_id}', rate={self.rate}, execution_rate={self.execution_rate}, rate_min={self.rate_min}, rate_max={self.rate_max}, trade_id={self.trade_id}, account_id={self.account_id}, account_name='{self.account_name}', offer_id={self.offer_id}, net_quantity={self.net_quantity}, buy_sell='{self.buy_sell}', stage='{self.stage}', type='{self.type}', status='{self.status}', status_time='{self.status_time}', amount={self.amount}, lifetime={self.lifetime}, at_market={self.at_market}, trail_step={self.trail_step}, trail_rate={self.trail_rate}, time_in_force='{self.time_in_force}', account_kind={self.account_kind}, request_txt='{self.request_txt}', contingent_order_id={self.contingent_order_id}, contingency_type={self.contingency_type}, primary_id='{self.primary_id}', origin_amount={self.origin_amount}, filled_amount={self.filled_amount}, working_indicator={self.working_indicator}, peg_type='{self.peg_type}', peg_offset={self.peg_offset}, peg_offset_min={self.peg_offset_min}, peg_offset_max={self.peg_offset_max}, expire_date='{self.expire_date}', value_date='{self.value_date}', parties='{self.parties}', side={self.side}, stop={self.stop}, limit={self.limit}, stop_order_id='{self.stop_order_id}', limit_order_id='{self.limit_order_id}', type_stop={self.type_stop}, type_limit={self.type_limit}, stop_trail_step={self.stop_trail_step}, stop_trail_rate={self.stop_trail_rate})"

#Wrapper classes for FXTrade and FXOrder
class FXTransactWrapper:
    def __init__(self, trades:FXTrades=None, orders:FXOrders=None):
        self.trades = trades if trades else FXTrades()
        self.orders = orders if orders else FXOrders()
    
    def to_dict(self):
        return {
            "trades": self.trades.to_dict()["trades"] if self.trades else None,
            "orders": self.orders.to_dict()["orders"] if self.orders else None
        }
    
    def add_trades(self, trades:FXTrades):
        self.trades = trades if trades else FXTrades()
    
    def add_trade(self, trade_data:FXTrade):
        self.trades.add_trade(trade_data)
    
    def add_orders(self, orders:FXOrders):
        self.orders = orders if orders else FXOrders()
    
    def add_order(self, order_data:FXOrder):
        self.orders.add_order(order_data)
    
    def tojson(self, indent=2):
        return json.dumps(self.to_dict(), indent=indent)
    
    def tojsonfile(self, filename:str):
        try:
            with open(filename, 'w') as f:
                f.write(self.tojson())
        except Exception as e:
            print(f"Error writing to file: {e}")
            
    
        



import os

import sys

sys.path.insert(0, os.path.abspath(os.path.dirname(__file__)))

import jgtcommon
import jgtwslhelper as wsl


def refreshPH(instrument:str, timeframe:str,quote_count:int=-1, quiet:bool=True,use_full:bool=False,verbose_level=0,tlid_range=None,keep_bid_ask=False):
  if not quiet:
    print(f"Refreshing {instrument} {timeframe}")
  try:
    wsl.getPH(instrument, timeframe,quote_count=quote_count, tlid_range=tlid_range, use_full=use_full,verbose_level=verbose_level,keep_bid_ask=keep_bid_ask)
  except Exception as e:
    print("Error in refreshPH")
    raise e

def parse_args():
    parser = jgtcommon.new_parser("WSL JGTFxCLI Test settings","Test loading custom settings and launch",enable_specified_settings=True)
    #parser=jgtcommon.add_settings_argument(parser)
    #parser=jgtcommon._preload_settings_from_args(parser)
    
    parser= jgtcommon.add_instrument_timeframe_arguments(parser)
    parser=jgtcommon.add_bars_amount_V2_arguments(parser)
    parser=jgtcommon.add_keepbidask_argument(parser)
    parser=jgtcommon.add_tlid_range_argument(parser)
    parser=jgtcommon.add_verbose_argument(parser)
    
    args = jgtcommon.parse_args(parser)
    return args

def main():
    args = parse_args()
    
    #settings = jgtcommon.load_settings(args)
    print("Settings loaded:")
    print(jgtcommon.settings)
    try:
        tlid_range_value = args.tlidrange if args.tlidrange else None
        refreshPH(args.instrument, args.timeframe,quote_count=args.quotescount, quiet=False,use_full=False,verbose_level=args.verbose,tlid_range=tlid_range_value,keep_bid_ask=args.keep_bid_ask)
    except Exception as e:
        print("Error in main")
        raise e
    
if __name__ == '__main__':
    main()
    



"""
Future use: JSON data for requests that we load from this to the cli args.

example:

--load_pattern CDS_RQ_JSON_NORM_FRESH

"""

PDS_RQ_BASE="""
{
  "dropna_volume": true,
  "keep_bid_ask": true,
  "mfi_flag": true,
  "balligator_flag": true,
  "talligator_flag": true,
}
"""

PDS_RQ_NORMAL="""
{
  "use_full": false,
  "use_fresh": false,
  "dropna_volume": true,
  "keep_bid_ask": true
}
"""
PDS_RQ_NORMAL_FRESH="""
{
  "use_full": false,
  "use_fresh": true,
  "dropna_volume": true,
  "keep_bid_ask": true
}
"""
PDS_RQ_FULL="""
{
  "use_full": true,
  "use_fresh": false,
  "dropna_volume": true,
  "keep_bid_ask": true
}
"""

PDS_RQ_FULL_FRESH="""
{
  "use_full": true,
  "use_fresh": false,
  "dropna_volume": true,
  "keep_bid_ask": true
}
"""

IDS_RQ_BASE="""
{
  "mfi_flag": true,
  "balligator_flag": true,
  "talligator_flag": true,
  "dropna_volume": true,
  "keep_bid_ask": true
}
"""

CDS_RQ_NORMAL="""
{
  "use_full": false,
  "use_fresh": false,
  "mfi_flag": true,
  "balligator_flag": true,
  "talligator_flag": true,
  "dropna_volume": true,
  "keep_bid_ask": true
}
"""
CDS_RQ_FULL="""
{
  "use_full": true,
  "use_fresh": false,
  "mfi_flag": true,
  "balligator_flag": true,
  "talligator_flag": true,
  "quotescount": -1,
  "dropna_volume": true,
  "keep_bid_ask": true
}
"""


CDS_RQ_FULL_FRESH="""
{
  "use_full": true,
  "use_fresh": true,
  "mfi_flag": true,
  "balligator_flag": true,
  "talligator_flag": true,
  "quotescount": -1,
  "dropna_volume": true,
  "keep_bid_ask": true
}
"""

CDS_RQ_NORM_FRESH="""
{
  "use_full": false,
  "use_fresh": true,
  "mfi_flag": true,
  "balligator_flag": true,
  "talligator_flag": true,
  "quotescount": -1,
  "dropna_volume": true,
  "keep_bid_ask": true
}
"""

CDS_RQ_NORM_FRESH__NO_MFI="""
{
  "use_full": false,
  "use_fresh": true,
  "mfi_flag": false,
  "balligator_flag": true,
  "talligator_flag": true,
  "quotescount": -1,
  "dropna_volume": false,
  "keep_bid_ask": true
}
"""



import unittest
import tempfile
import shutil
import os
import json
from jgtcommon import load_settings

class TestLoadSettings(unittest.TestCase):
  #just run the load_settings function
  def test_load_settings(self):
    settings=load_settings()
    self.assertTrue(settings is not None)

if __name__ == '__main__':
    unittest.main()


"""
jgtutils package
"""

version='0.2.43'

import sys
import os

sys.path.insert(0, os.path.abspath(os.path.dirname(__file__)))

from jgtos import (tlid_range_to_jgtfxcon_start_end_str,
                   tlid_range_to_start_end_datetime)

from jgtutils import jgtcommon as common
from jgtutils import jgtos as jos
from jgtutils import jgtpov as pov
from jgtutils import jgtwslhelper as wsl
from jgtutils.jgtcommon import readconfig
from jgtutils.jgtpov import calculate_tlid_range as get_tlid_range


def load_logging():
  from jgtutils import jgtlogging as jlog




OFFERS_CSV_DATA="""18,CAD/JPY
1010,SPX500
2020,WHEATF
3008,2USNote
3,GBP/USD
36,EUR/NOK
14,EUR/AUD
3004,5USNote
17,AUD/JPY
32,EUR/SEK
21,GBP/NZD
2003,SOYF
1035,EUSTX50
1002,ESP35
89,NZD/CHF
96,EUR/HUF
1013,US30
3011,FED30D
4,USD/CHF
22,GBP/AUD
13,GBP/CHF
1001,AUS200
15,EUR/CAD
90,CAD/CHF
1012,UK100
4001,XAU/USD
3005,10USNote
39,AUD/CHF
91,NZD/CAD
2,USD/JPY
30,USD/SEK
5,EUR/CHF
38,USD/MXN
83,USD/TRY
3010,Schatz
1008,NAS100
97,USD/HUF
1,EUR/USD
20,GBP/CAD
16,AUD/CAD
1060,US2000
1004,GER30
12,CHF/JPY
50,USD/HKD
37,USD/NOK
2001,USOil
1058,USDOLLAR
1007,JPN225
98,TRY/JPY
71,ZAR/JPY
9,EUR/GBP
47,USD/ZAR
105,USD/CNH
4002,XAG/USD
2021,CORNF
6,AUD/USD
11,GBP/JPY
1003,FRA40
3001,Bund
2015,NGAS
2002,UKOil
8,NZD/USD
1005,HKG33
3012,EURIBOR3M
19,NZD/JPY
40,EUR/NZD
28,AUD/NZD
3014,SONIA3M
3009,Bobl
1016,Copper
87,EUR/TRY
7,USD/CAD
10,EUR/JPY"""

def offer_id_to_instrument(offer_id):
    for line in OFFERS_CSV_DATA.split("\n"):
        if line.startswith(str(offer_id) + ","):
            return line.split(",")[1]
    return None

def instrument_to_offer_id(symbol):
    for line in OFFERS_CSV_DATA.split("\n"):
        if line.split(",")[1] == symbol:
            return int(line.split(",")[0])
    return None


import re

from setuptools import find_packages, setup


def read_version():
    with open("jgtutils/__init__.py") as f:
        content=f.read()
        version_match = re.search(r"version=['\"]([^'\"]*)['\"]", content)
        return version_match.group(1)

version = read_version()

INSTALL_REQUIRES = [
    'tlid>=0.1.15',
]
setup(
    name='jgtutils',
    version=version,
    packages=find_packages(),
    install_requires=INSTALL_REQUIRES,
    description='A utility package common to other JGT projects.',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    author='Guillaume Isabelle',
    author_email='jgi@jgwill.com',
    url='https://github.com/jgwill/jgtutils',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],    
    # entry_points={
    #     'console_scripts': ['jgtutr=jgtutils.cli_tlid_range:main'],
    # },
    keywords='utilities',
    project_urls={
        'Bug Reports': 'https://github.com/jgwill/jgtutils/issues',
        'Source': 'https://github.com/jgwill/jgtutils',
    },
)




NOT_FRESH_FLAG_ARGNAME = 'notfresh'
NOT_FRESH_FLAG_ARGNAME_ALIAS='old'
FRESH_FLAG_ARGNAME = 'fresh'
FRESH_FLAG_ARGNAME_ALIAS = 'new'

LARGEST_FRACTAL_PERIOD_ARGNAME="largest_fractal_period"
LARGEST_FRACTAL_PERIOD_ARGNAME_ALIAS="lfp"
LARGEST_FRACTAL_PERIOD_ARGTYPE=int

BALLIGATOR_FLAG_ARGNAME = 'balligator_flag'
BALLIGATOR_FLAG_ARGNAME_ALIAS = 'ba'
BALLIGATOR_PERIOD_JAWS_ARGNAME = "balligator_period_jaws"
BALLIGATOR_PERIOD_JAWS_ARGNAME_ALIAS = "bjaw"
BALLIGATOR_PERIOD_JAWS_ARGTYPE=int
BALLIGATOR_PERIOD_TEETH_ARGNAME="balligator_period_teeth"
BALLIGATOR_PERIOD_TEETH_ARGNAME_ALIAS="bteeth"
BALLIGATOR_PERIOD_TEETH_ARGTYPE=int
BALLIGATOR_PERIOD_LIPS_ARGNAME="balligator_period_lips"
BALLIGATOR_PERIOD_LIPS_ARGNAME_ALIAS="blips"
BALLIGATOR_PERIOD_LIPS_ARGTYPE=int


TALLIGATOR_FLAG_ARGNAME = 'talligator_flag'
TALLIGATOR_FLAG_ARGNAME_ALIAS = 'ta'
TALLIGATOR_PERIOD_JAWS_ARGNAME = "talligator_period_jaws"
TALLIGATOR_PERIOD_JAWS_ARGNAME_ALIAS = "tjaw"
TALLIGATOR_PERIOD_JAWS_ARGTYPE=int
TALLIGATOR_PERIOD_TEETH_ARGNAME="talligator_period_teeth"
TALLIGATOR_PERIOD_TEETH_ARGNAME_ALIAS="tteeth"
TALLIGATOR_PERIOD_TEETH_ARGTYPE=int
TALLIGATOR_PERIOD_LIPS_ARGNAME="talligator_period_lips"
TALLIGATOR_PERIOD_LIPS_ARGNAME_ALIAS="tlips"
TALLIGATOR_PERIOD_LIPS_ARGTYPE=int

MFI_FLAG_ARGNAME = 'mfi_flag'
NO_MFI_FLAG_ARGNAME = 'no_mfi_flag'
MFI_FLAG_ARGNAME_ALIAS = 'mfi'
NO_MFI_FLAG_ARGNAME_ALIAS = 'nomfi'

#gator_oscillator_flag,go
GATOR_OSCILLATOR_FLAG_ARGNAME = 'gator_oscillator_flag'
GATOR_OSCILLATOR_FLAG_ARGNAME_ALIAS = 'go'

#keepbidask_flag,kba
KEEP_BID_ASK_FLAG_ARGNAME = 'keepbidask'
KEEP_BID_ASK_FLAG_ARGNAME_ALIAS = 'kba'
REMOVE_BID_ASK_FLAG_ARGNAME = 'rmbidask'
REMOVE_BID_ASK_FLAG_ARGNAME_ALIAS = 'rmba'

#quotescount
QUOTES_COUNT_ARGNAME = 'quotescount'
QUOTES_COUNT_ARGNAME_ALIAS = 'c'
#full flag
FULL_FLAG_ARGNAME = 'full'
FULL_FLAG_ARGNAME_ALIAS = 'uf'
NOT_FULL_FLAG_ARGNAME = 'notfull'
NOT_FULL_FLAG_ARGNAME_ALIAS = 'nf'

#dropna_volume
DROPNA_VOLUME_FLAG_ARGNAME = 'dropna_volume'
DROPNA_VOLUME_FLAG_ARGNAME_ALIAS = 'dv'
#dont_dropna_volume
DONT_DROPNA_VOLUME_FLAG_ARGNAME = 'dont_dropna_volume'
DONT_DROPNA_VOLUME_FLAG_ARGNAME_ALIAS = 'ddv'

ARG_GROUP_POV_TITLE="POV"
ARG_GROUP_POV_DESCRIPTION="Point of view"
ARG_GROUP_RANGE_TITLE="DTRange"
ARG_GROUP_RANGE_DESCRIPTION="Date and range selection"

ARG_GROUP_INDICATOR_TITLE = 'Indicators'
ARG_GROUP_INDICATOR_DESCRIPTION = 'Indicators flags'

ARG_GROUP_BARS_TITLE="Bars"
ARG_GROUP_BARS_DESCRIPTION="Bars flags"

ARG_GROUP_CLEANUP_TITLE = 'Cleanup'
ARG_GROUP_CLEANUP_DESCRIPTION = 'Cleanup data'

ARG_GROUP_VERBOSITY_TITLE="Verbosity"
ARG_GROUP_VERBOSITY_DESCRIPTION="control the verbosity of the output"

ARG_GROUP_OUTPUT_TITLE="Output"
ARG_GROUP_OUTPUT_DESCRIPTION="Output arguments"

ARG_GROUP_INTERACTION_TITLE="Interaction"
ARG_GROUP_INTERACTION_DESCRIPTION="Interaction arguments"


TLID_RANGE_ARGNAME='range'
TLID_RANGE_ARG_DEST='tlidrange'
TLID_RANGE_ARGNAME_ALIAS=''
TLID_RANGE_HELP_STRING='TLID range are two dates formated: YYMMDDHHMM_YYMMDDHHMM.'



DEMO_FLAG_ARGNAME='demo'

SETTING_ARGNAME='settings'
SETTING_ARGNAME_ALIAS='ls'


RATE_ARGNAME='rate'
RATE_ARGNAME_ALIAS='r'

BUYSELL_ARGNAME='bs'
BUYSELL_ARGNAME_ALIAS='d'

STOP_ARGNAME='stop'
STOP_ARGNAME_ALIAS='x' 

LOTS_ARGNAME='lots'
LOTS_ARGNAME_ALIAS='n'

ORDERID_ARGNAME='orderid'
ORDERID_ARGNAME_ALIAS='id'

TRADEID_ARGNAME='tradeid'
TRADEID_ARGNAME_ALIAS='tid'

ACCOUNT_ARGNAME='account'





# Copyright 2019 Gehtsoft USA LLC
# Copyright 2023 JGWill (extended/variations)

# Licensed under the license derived from the Apache License, Version 2.0 (the "License"); 
# you may not use this file except in compliance with the License.

# You may obtain a copy of the License at

# http://fxcodebase.com/licenses/open-source/license.html

# Unless required by applicable law or agreed to in writing, software 
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import json
import os
import sys
import traceback
#import logging
from datetime import datetime, time
from enum import Enum
from typing import List

import tlid

#------------------------#

# common.py



sys.path.insert(0, os.path.abspath(os.path.dirname(__file__)))

from jgtos import (tlid_dt_to_string, tlid_range_to_jgtfxcon_start_end_str,
                   tlid_range_to_start_end_datetime, tlidmin_to_dt)

from jgtcliconstants import (ACCOUNT_ARGNAME, ARG_GROUP_BARS_DESCRIPTION,
                                      ARG_GROUP_BARS_TITLE,
                                      ARG_GROUP_CLEANUP_DESCRIPTION,
                                      ARG_GROUP_CLEANUP_TITLE,
                                      ARG_GROUP_INDICATOR_DESCRIPTION,
                                      ARG_GROUP_INDICATOR_TITLE,
                                      ARG_GROUP_INTERACTION_DESCRIPTION,
                                      ARG_GROUP_INTERACTION_TITLE,
                                      ARG_GROUP_OUTPUT_DESCRIPTION,
                                      ARG_GROUP_OUTPUT_TITLE,
                                      ARG_GROUP_POV_DESCRIPTION,
                                      ARG_GROUP_POV_TITLE,
                                      ARG_GROUP_RANGE_DESCRIPTION,
                                      ARG_GROUP_RANGE_TITLE,
                                      ARG_GROUP_VERBOSITY_DESCRIPTION,
                                      ARG_GROUP_VERBOSITY_TITLE,
                                      BALLIGATOR_FLAG_ARGNAME,
                                      BALLIGATOR_FLAG_ARGNAME_ALIAS, BUYSELL_ARGNAME, BUYSELL_ARGNAME_ALIAS,
                                      DONT_DROPNA_VOLUME_FLAG_ARGNAME,
                                      DONT_DROPNA_VOLUME_FLAG_ARGNAME_ALIAS,
                                      DROPNA_VOLUME_FLAG_ARGNAME,
                                      DROPNA_VOLUME_FLAG_ARGNAME_ALIAS,
                                      FRESH_FLAG_ARGNAME,
                                      FRESH_FLAG_ARGNAME_ALIAS,
                                      FULL_FLAG_ARGNAME,
                                      FULL_FLAG_ARGNAME_ALIAS,
                                      GATOR_OSCILLATOR_FLAG_ARGNAME,
                                      GATOR_OSCILLATOR_FLAG_ARGNAME_ALIAS,
                                      KEEP_BID_ASK_FLAG_ARGNAME,
                                      KEEP_BID_ASK_FLAG_ARGNAME_ALIAS, LOTS_ARGNAME, LOTS_ARGNAME_ALIAS,
                                      MFI_FLAG_ARGNAME, MFI_FLAG_ARGNAME_ALIAS,
                                      NO_MFI_FLAG_ARGNAME,
                                      NO_MFI_FLAG_ARGNAME_ALIAS,
                                      NOT_FRESH_FLAG_ARGNAME,
                                      NOT_FRESH_FLAG_ARGNAME_ALIAS,
                                      NOT_FULL_FLAG_ARGNAME,
                                      NOT_FULL_FLAG_ARGNAME_ALIAS, ORDERID_ARGNAME, ORDERID_ARGNAME_ALIAS,
                                      QUOTES_COUNT_ARGNAME,
                                      QUOTES_COUNT_ARGNAME_ALIAS, RATE_ARGNAME, RATE_ARGNAME_ALIAS,
                                      REMOVE_BID_ASK_FLAG_ARGNAME,
                                      REMOVE_BID_ASK_FLAG_ARGNAME_ALIAS, STOP_ARGNAME, STOP_ARGNAME_ALIAS,
                                      TALLIGATOR_FLAG_ARGNAME,
                                      TALLIGATOR_FLAG_ARGNAME_ALIAS,
                                      TLID_RANGE_ARG_DEST, TLID_RANGE_ARGNAME,
                                      TLID_RANGE_ARGNAME_ALIAS,
                                      TLID_RANGE_HELP_STRING, TRADEID_ARGNAME, TRADEID_ARGNAME_ALIAS)

args:argparse.Namespace=None # Default args when we are done parsing
try :
    import __main__

    # logging.basicConfig(filename='{0}.log'.format(__main__.__file__), level=logging.INFO,
    #                 format='%(asctime)s %(levelname)s %(message)s', datefmt='%m.%d.%Y %H:%M:%S')
    # console = logging.StreamHandler(sys.stdout)
    # console.setLevel(logging.INFO)
    # logging.getLogger('').addHandler(console)
    

except:
    #print('logging failed - dont worry')
    pass

try :    
    #if __main__ has a .parser then set the default parser to that
    if hasattr(__main__,'parser'):
        default_parser=__main__.parser
    else:
        if hasattr(__main__,'default_parser'):
            default_parser=__main__.default_parser
        else: 
            if hasattr(__main__,'__parser__'):
                default_parser=__main__.__parser__
            else:
                default_parser = argparse.ArgumentParser(description='JGWill Trading Utilities')
except:
    default_parser = argparse.ArgumentParser(description='JGWill Trading Utilities')
    pass



settings: dict = None

# try:
#     #indicator's group
#     indicator_group = default_parser.add_argument_group(INDICATOR_GROUP_TITLE, 'Indicators to use in the processing.')
#     #indicator_group = _get_group_by_title(default_parser, INDICATOR_GROUP_TITLE)
# except:
#     pass

def _load_settings_from_path(path):
    if os.path.exists(path):
        with open(path, 'r') as f:
            return json.load(f)
    return {}

def load_settings(custom_path=None,old=None):
    system_settings_path = os.path.join('/etc', 'jgt', 'settings.json')
    home_settings_path = os.path.join(os.path.expanduser('~'), '.jgt', 'settings.json')
    current_settings_path = os.path.join(os.getcwd(), '.jgt', 'settings.json')
    
    _settings={}
    if old is not None:
        _settings=old
    
    #load json from env JGT_SETTINGS if exist
    if 'JGT_SETTINGS_SYSTEM' in os.environ:
        print("JGT_SETTINGS_SYSTEM found")
        env_settings=json.loads(os.environ['JGT_SETTINGS_SYSTEM'])
        update_settings(_settings, env_settings)
        
    
    system_settings=_load_settings_from_path(system_settings_path)
     # Merge settings
    update_settings(_settings, system_settings)

    
    if 'JGT_SETTINGS' in os.environ:
        print("JGT_SETTINGS found")
        env_settings=json.loads(os.environ['JGT_SETTINGS'])
        _settings.update(env_settings)
    
    user_settings = _load_settings_from_path(home_settings_path)
     # Merge settings, with user directory settings taking precedence
    update_settings(_settings, user_settings)
    
    current_settings = _load_settings_from_path(current_settings_path)
    
    # Merge settings, with current directory settings taking precedence
    update_settings(_settings, current_settings)
    
    if custom_path is not None and custom_path != '':
        custom_settings = _load_settings_from_path(custom_path)
        update_settings(_settings, custom_settings)
    
    if 'JGT_SETTINGS_PROCESS' in os.environ:
        print("JGT_SETTINGS PROCESS found")
        env_settings=json.loads(os.environ['JGT_SETTINGS_PROCESS'])
        update_settings(_settings,env_settings )
    
    _settings_loaded(_settings)
    
    return _settings

def update_settings(old_settings, new_settings,keys=['patterns']):
    #if our old settings has a key in our keys list, then we will update it on their own (meaning we will not merge it directly but update it independently)
    for key in keys:
        if key in old_settings:
            tst_key_value_old=old_settings[key]
            if key in new_settings:
                old_settings[key].update(new_settings[key])
                #new_settings.pop(key)
                tst_key_value_new=old_settings[key]
                #remove the key from the new settings
                new_settings.pop(key)
                #print("Updated key: "+key)
    old_settings.update(new_settings)
    #print("Updated settings")

def _settings_loaded(_settings):
    return

def get_settings()->dict:
    global settings
    if settings is None:
        settings = load_settings()
    return settings

def load_arg_default_from_settings(argname:str,default_value,alias:str=None):
    global settings
    if settings is None:
        settings=load_settings()
    
    _value = settings.get(argname,default_value)
    if alias is not None and _value==default_value:
        _value = settings.get(alias,default_value) #try alias might be used
    return _value

def add_settings_argument(parser: argparse.ArgumentParser=None)->argparse.ArgumentParser:
    global default_parser
    if parser is None:
        parser=default_parser
    parser.add_argument('-'+SETTING_ARGNAME_ALIAS,'--'+SETTING_ARGNAME,
                       type=str,
                        help='Load settings from a specific settings file (overrides default settings (/etc/jgt/settings.json and HOME/.jgt/settings.json and .jgt/settings.json)).',
                        required=False)
    return parser

def _preload_settings_from_args(parser: argparse.ArgumentParser=None):
    global default_parser,settings
    if parser is None:
        parser=default_parser
    
    args, unknown = parser.parse_known_args()
    settings = load_settings(getattr(args,SETTING_ARGNAME,None))
    
    return parser

def new_parser(description: str,epilog: str=None,prog: str=None,enable_specified_settings=True)->argparse.ArgumentParser:
    global default_parser
    default_parser = argparse.ArgumentParser(description=description,epilog=epilog,prog=prog)
    
    if enable_specified_settings:
        default_parser=add_settings_argument(default_parser)
        if not '--help' in sys.argv:
            default_parser=_preload_settings_from_args(default_parser)
    
    return default_parser

# Get a group by its title
def _get_group_by_title(parser, title,description=""):
    for group in parser._action_groups:
        if group.title == title:
            return group
    #create it
    return parser.add_argument_group(title, description)


def init_default_parser(description: str):
    global default_parser
    default_parser = argparse.ArgumentParser(description=description)
    return default_parser




def _add_a_flag_helper(_description:str,  _argname_alias:str, _argname_full:str, parser: argparse.ArgumentParser,_action_value="store_true",group_title="",group_description="",load_default_from_settings=True,flag_default_value=False):


    __alias_cmd_prefix = "-"
    __full_arg_prefix = "--"
    
    __flag_setting_value=load_arg_default_from_settings(_argname_full,flag_default_value)
    
    _argname_alias = __alias_cmd_prefix+_argname_alias
    _argname_full = __full_arg_prefix+_argname_full
    if group_title=="":
        parser.add_argument(
        _argname_alias,
        _argname_full,
        action=_action_value,
        help=_description,
        default=__flag_setting_value
        )
    else:
        #try get group name or create it.
        group = _get_group_by_title(parser, group_title,group_description)
        group.add_argument(
            _argname_alias,
            _argname_full,
            action=_action_value,
            help=_description,
            default=__flag_setting_value
        )
    
    return parser






def add_main_arguments(parser: argparse.ArgumentParser=None)->argparse.ArgumentParser:
    global default_parser
    if parser is None:
        parser=default_parser
        
    parser.add_argument('--login',
                        metavar="LOGIN",
                        required=True,
                        help='Your user name.')

    parser.add_argument('--password',
                        metavar="PASSWORD",
                        required=True,
                        help='Your password.')

    parser.add_argument('--urlserver',
                        metavar="URL",
                        required=True,
                        help='The server URL. For example,\
                                 https://www.fxcorporate.com/Hosts.jsp.')

    parser.add_argument('--connection',
                        metavar="CONNECTION",
                        required=True,
                        help='The connection name. For example, \
                                 "Demo" or "Real".')


    parser.add_argument('-session',
                        help='The database name. Required only for users who\
                                 have accounts in more than one database.\
                                 Optional parameter.')

    parser.add_argument('-pin',
                        help='Your pin code. Required only for users who have \
                                 a pin. Optional parameter.')
    return parser

def add_candle_open_price_mode_argument(parser: argparse.ArgumentParser=None)->argparse.ArgumentParser:
    global default_parser
    if parser is None:
        parser=default_parser
    parser.add_argument('--openpricemode',
                        metavar="CANDLE_OPEN_PRICE_MODE",
                        default="prev_close",
                        help='Ability to set the open price candles mode. \
                        Possible values are first_tick, prev_close. For more information see description \
                        of O2GCandleOpenPriceMode enumeration. Optional parameter.')
    return parser

from jgtcliconstants import (DEMO_FLAG_ARGNAME)

def add_demo_flag_argument(parser: argparse.ArgumentParser=None,load_default_from_settings=True,flag_default_value=False)->argparse.ArgumentParser:
    global default_parser
    if parser is None:
        parser=default_parser
    default_value = load_arg_default_from_settings(DEMO_FLAG_ARGNAME,flag_default_value) if load_default_from_settings else flag_default_value
    parser.add_argument('--demo',
                        action='store_true',
                        help='Use the demo server. Optional parameter.',
                        default=default_value)
    return parser

def add_instrument_timeframe_arguments(parser: argparse.ArgumentParser=None, timeframe: bool = True,add_IndicatorPattern=False,load_instrument_from_settings=True,load_timeframe_from_settings=True)->argparse.ArgumentParser:
    
    global default_parser
    if parser is None:
        parser=default_parser
    pov_group=_get_group_by_title(parser,ARG_GROUP_POV_TITLE,ARG_GROUP_POV_DESCRIPTION)
    instrument_setting_value=None if not load_instrument_from_settings else load_arg_default_from_settings("instrument",None,alias="i")
    pov_group.add_argument('-i','--instrument',
                        metavar="INSTRUMENT",
                        help='An instrument which you want to use in sample. \
                                  For example, "EUR/USD".',
                                  default=instrument_setting_value)

    if timeframe:
        timeframe_setting_value=None if not load_timeframe_from_settings else load_arg_default_from_settings("timeframe",None,alias="t")
        pov_group.add_argument('-t','--timeframe',
                            metavar="TIMEFRAME",
                            help='Time period which forms a single candle. \
                                      For example, m1 - for 1 minute, H1 - for 1 hour.',
                                      default=timeframe_setting_value)
    if add_IndicatorPattern:
        parser.add_argument('-ip',
                        metavar="IndicatorPattern",
                        required=False,
                        help='The indicator Pattern. For example, \
                                 "AOAC","JTL,"JTLAOAC","JTLAOAC","AOACMFI".')
    return parser

def add_instrument_standalone_argument(parser: argparse.ArgumentParser=None,load_from_settings=True,required=False)->argparse.ArgumentParser:
    global default_parser
    if parser is None:
        parser=default_parser
    instrument_value=load_arg_default_from_settings("instrument",None,"i") if load_from_settings else None
    parser.add_argument('-i','--instrument',
                        metavar="INSTRUMENT",
                        help='An instrument which you want to use in sample. \
                                  For example, "EUR/USD".',
                                  default=instrument_value,
                                  required=required)
    return parser

TIMEFRAME_DEFAULT_STANDALONE = "D1"
def add_timeframe_standalone_argument(parser: argparse.ArgumentParser=None,load_from_settings=True,required=False,load_default_timeframe=False)->argparse.ArgumentParser:
    global default_parser
    if parser is None:
        parser=default_parser
    
    tf_default_value=load_arg_default_from_settings("timeframe",TIMEFRAME_DEFAULT_STANDALONE if load_default_timeframe else None,"t") if load_from_settings else None
    
    parser.add_argument("-t","--timeframe", help="Timeframe", default=tf_default_value, required=required)
    return parser


def add_direction_buysell_arguments(parser: argparse.ArgumentParser=None,load_from_settings=True)->argparse.ArgumentParser:
    global default_parser
    if parser is None:
        parser=default_parser
    
    bs_value=load_arg_default_from_settings(BUYSELL_ARGNAME,None,BUYSELL_ARGNAME_ALIAS) if load_from_settings else None
    parser.add_argument('-'+BUYSELL_ARGNAME_ALIAS,'--'+BUYSELL_ARGNAME, metavar="TYPE", required=True,
                        help='The order direction. Possible values are: B - buy, S - sell.',default=bs_value)
    return parser

def add_rate_arguments(parser: argparse.ArgumentParser=None,load_from_settings=True)->argparse.ArgumentParser:
    global default_parser
    if parser is None:
        parser=default_parser
    
    rate_value=load_arg_default_from_settings(RATE_ARGNAME,None,RATE_ARGNAME_ALIAS) if load_from_settings else None
    parser.add_argument('-'+RATE_ARGNAME_ALIAS,'--'+RATE_ARGNAME, metavar="RATE", required=True, type=float,
                            help='Desired price of an entry order.',
                            default=rate_value)
    return parser

def add_stop_arguments(parser: argparse.ArgumentParser=None,load_from_settings=True)->argparse.ArgumentParser:
    global default_parser
    if parser is None:
        parser=default_parser
    
    stop_value=load_arg_default_from_settings(STOP_ARGNAME,None,STOP_ARGNAME_ALIAS) if load_from_settings else None
    
    parser.add_argument('-'+STOP_ARGNAME_ALIAS,'-'+STOP_ARGNAME,'--'+STOP_ARGNAME, metavar="STOP", required=True, type=float,
                            help='Desired price of the stop order.',
                            default=stop_value)
    return parser

def add_lots_arguments(parser,load_from_settings=True,default_value = 1):
    global default_parser
    if parser is None:
        parser=default_parser
    
    lots_value=load_arg_default_from_settings(LOTS_ARGNAME,default_value,LOTS_ARGNAME_ALIAS) if load_from_settings else default_value
    parser.add_argument('-'+LOTS_ARGNAME_ALIAS,'-'+LOTS_ARGNAME,'--'+LOTS_ARGNAME, metavar="LOTS", default=lots_value, type=int,
                            help='Trade amount in lots.')

def add_direction_rate_lots_arguments(parser: argparse.ArgumentParser=None, direction: bool = True, rate: bool = True,
                                      lots: bool = True, stop: bool = True,load_from_settings=True,lots_default_value=1)->argparse.ArgumentParser:
    global default_parser
    if parser is None:
        parser=default_parser

    if direction:
        add_direction_buysell_arguments(parser,load_from_settings)
    if rate:
        add_rate_arguments(parser,load_from_settings)
    if lots:
        add_lots_arguments(parser,load_from_settings,lots_default_value)
    if stop:
        add_stop_arguments(parser,load_from_settings)
    
    return parser


def add_orderid_arguments(parser: argparse.ArgumentParser=None,load_from_settings=True,required=True)->argparse.ArgumentParser:
    global default_parser
    if parser is None:
        parser=default_parser
    orderid_value=load_arg_default_from_settings(ORDERID_ARGNAME,None,ORDERID_ARGNAME_ALIAS) if load_from_settings else None
    parser.add_argument('-'+ORDERID_ARGNAME_ALIAS,'--'+ORDERID_ARGNAME, metavar="OrderID", required=required,
                        help='The order identifier.',
                        default=orderid_value)
    return parser

def add_tradeid_arguments(parser: argparse.ArgumentParser=None,load_from_settings=True,required=False)->argparse.ArgumentParser:
    global default_parser
    if parser is None:
        parser=default_parser
    tradeid_value=load_arg_default_from_settings(TRADEID_ARGNAME,None,TRADEID_ARGNAME_ALIAS) if load_from_settings else None
    parser.add_argument('-'+TRADEID_ARGNAME_ALIAS,'--'+TRADEID_ARGNAME, metavar="TradeID", required=required,
                        help='The trade identifier.',
                        default=tradeid_value)
    return parser


def add_account_arguments(parser: argparse.ArgumentParser=None,load_from_settings=True,required=False)->argparse.ArgumentParser:
    global default_parser
    if parser is None:
        parser=default_parser
    account_value=load_arg_default_from_settings(ACCOUNT_ARGNAME,None) if load_from_settings else None
    parser.add_argument('-'+ACCOUNT_ARGNAME, metavar="ACCOUNT",
                        help='An account which you want to use in sample.',default=account_value,required=required)
    return parser


def str_to_datetime(date_str):
    formats = ['%m.%d.%Y %H:%M:%S', '%Y-%m-%d %H:%M:%S', '%Y/%m/%d %H:%M:%S', '%Y/%m/%d', '%Y-%m-%d']
    
    for fmt in formats:
        try:
            return datetime.datetime.strptime(date_str, fmt)
        except ValueError:
            continue
    return None

def valid_datetime(check_future: bool):
    def _valid_datetime(str_datetime: str):
        date_format = '%m.%d.%Y %H:%M:%S'
        try:
            result = datetime.datetime.strptime(str_datetime, date_format).replace(
                tzinfo=datetime.timezone.utc)
            if check_future and result > datetime.datetime.utcnow().replace(tzinfo=datetime.timezone.utc):
                msg = "'{0}' is in the future".format(str_datetime)
                raise argparse.ArgumentTypeError(msg)
            return result
        except ValueError:
            now = datetime.datetime.now()
            msg = "The date '{0}' is invalid. The valid data format is '{1}'. Example: '{2}'".format(
                str_datetime, date_format, now.strftime(date_format))
            raise argparse.ArgumentTypeError(msg)
    return _valid_datetime


def add_tlid_date_V2_arguments(parser: argparse.ArgumentParser=None)->argparse.ArgumentParser:
    global default_parser
    if parser is None:
        parser=default_parser
    
    group1 = parser.add_argument_group('Group 1 (TLID Range)')
    g1x=group1.add_mutually_exclusive_group()
    g1x.add_argument('-'+TLID_RANGE_ARGNAME_ALIAS, '--'+TLID_RANGE_ARGNAME, type=str, required=False, dest=TLID_RANGE_ARG_DEST,
                        help=TLID_RANGE_HELP_STRING)
    g2x=g1x.add_mutually_exclusive_group()
    
    raise Exception("Not implemented - Complicated")
    
    #group1.
    #dt_range_group=parser.add_mutually_exclusive_group()
    #dt_range_group.add_argument('-r', '--range', type=str, required=False, dest='tlidrange',
    #                    help='TLID range in the format YYMMDDHHMM_YYMMDDHHMM.')
    # Second group of arguments
    group2 = parser.add_argument_group('Group 2 (Dates)')
    group2.add_argument('-s', '--datefrom', metavar='"m.d.Y H:M:S"',
                        help='Date/time from which you want to receive historical prices.',
                        type=valid_datetime(True),required=False)
    group2.add_argument('-e', '--dateto', metavar='"m.d.Y H:M:S"',
                        help='Datetime until which you want to receive historical prices.',
                        type=valid_datetime(False),required=False)
    
    # Exclusivity between the two groups
    group1_xor_group2 = parser.add_mutually_exclusive_group()
    group1_xor_group2.add_argument('-r', '--range', dest='tlidrange', action='store_true')
    
    group2x=group1_xor_group2.add_mutually_exclusive_group()
    group2x.add_argument('-s', '--datefrom', dest='tlidrange', action='store_false')
    group2x.add_argument('-e', '--dateto', dest='tlidrange', action='store_false')
    
    
    return parser


def add_tlid_range_argument(parser: argparse.ArgumentParser=None)->argparse.ArgumentParser:
    global default_parser
    if parser is None:
        parser=default_parser
    #print("Tlid range active")
    group_range=_get_group_by_title(parser,ARG_GROUP_RANGE_TITLE,ARG_GROUP_RANGE_DESCRIPTION)
    group_range.add_argument('-'+TLID_RANGE_ARGNAME_ALIAS, '--'+TLID_RANGE_ARGNAME, type=str, required=False, dest=TLID_RANGE_ARG_DEST,
                        help=TLID_RANGE_HELP_STRING)
    return parser

def add_date_arguments(parser: argparse.ArgumentParser=None, date_from: bool = True, date_to: bool = True):
    global default_parser
    if parser is None:
        parser=default_parser
    
    group_range=_get_group_by_title(parser,ARG_GROUP_RANGE_TITLE,ARG_GROUP_RANGE_DESCRIPTION)
    if date_from:
        group_range.add_argument('-s','--datefrom',
                            metavar="\"m.d.Y H:M:S\"",
                            help='Date/time from which you want to receive\
                                      historical prices. If you leave this argument as it \
                                      is, it will mean from last trading day. Format is \
                                      "m.d.Y H:M:S". Optional parameter.',
                            type=valid_datetime(True)
                            )
    if date_to:
        group_range.add_argument('-e','--dateto',
                            metavar="\"m.d.Y H:M:S\"",
                            help='Datetime until which you want to receive \
                                      historical prices. If you leave this argument as it is, \
                                      it will mean to now. Format is "m.d.Y H:M:S". \
                                      Optional parameter.',
                            type=valid_datetime(False)
        )
    return parser


def add_report_date_arguments(parser: argparse.ArgumentParser=None, date_from: bool = True, date_to: bool = True):
    global default_parser
    if parser is None:
        parser=default_parser
    group_range=_get_group_by_title(parser,ARG_GROUP_RANGE_TITLE,ARG_GROUP_RANGE_DESCRIPTION)
    if date_from:
        group_range.add_argument('-s','--datefrom',
                            metavar="\"m.d.Y H:M:S\"",
                            help='Datetime from which you want to receive\
                                      combo account statement report. If you leave this argument as it \
                                      is, it will mean from last month. Format is \
                                      "m.d.Y H:M:S". Optional parameter.',
                            type=valid_datetime(True)
                            )
    if date_to:
        group_range.add_argument('-e','--dateto',
                            metavar="\"m.d.Y H:M:S\"",
                            help='Datetime until which you want to receive \
                                      combo account statement report. If you leave this argument as it is, \
                                      it will mean to now. Format is "m.d.Y H:M:S". \
                                      Optional parameter.',
                            type=valid_datetime(True)
        )
    return parser


def add_max_bars_arguments(parser: argparse.ArgumentParser=None)->argparse.ArgumentParser:
    global default_parser
    if parser is None:
        parser=default_parser
    
    group_bars=_get_group_by_title(parser,ARG_GROUP_BARS_TITLE,ARG_GROUP_BARS_DESCRIPTION)
    print("DEPRECATION: Use: add_bars_amount_V2_arguments intead of add_max_bars_arguments and add_use_full_argument")
    group_bars.add_argument('-'+QUOTES_COUNT_ARGNAME_ALIAS,'--'+QUOTES_COUNT_ARGNAME,
                        metavar="MAX",
                        default=-1,
                        type=int,
                        help='Max number of bars. 0 - Not limited')
    
    return parser

def add_bars_amount_V2_arguments(parser: argparse.ArgumentParser=None,load_from_settings=True)->argparse.ArgumentParser:
    global default_parser
    if parser is None:
        parser=default_parser
    #help='Specify the number of bars to download or use the full number of bars available from the store.'
    bars_group=_get_group_by_title(parser,ARG_GROUP_BARS_TITLE,ARG_GROUP_BARS_DESCRIPTION)
    #g=parser.add_argument_group('Bars Amount', 'Specify the number of bars to download or use the full number of bars available from the store.')
    bars_exclusive_subgroup=bars_group.add_mutually_exclusive_group()
    
    quotescount_value=load_arg_default_from_settings(QUOTES_COUNT_ARGNAME,-1,QUOTES_COUNT_ARGNAME_ALIAS) if load_from_settings else -1
    bars_exclusive_subgroup.add_argument('-'+QUOTES_COUNT_ARGNAME_ALIAS,'--'+QUOTES_COUNT_ARGNAME,
                        metavar="MAX",
                        default=quotescount_value,
                        type=int,
                        help='Max number of bars. 0 - Not limited')
    g_full_notfull=bars_exclusive_subgroup.add_mutually_exclusive_group()
    use_full_value=load_arg_default_from_settings(FULL_FLAG_ARGNAME,False,FULL_FLAG_ARGNAME_ALIAS) if load_from_settings else False
    g_full_notfull.add_argument('-'+FULL_FLAG_ARGNAME_ALIAS,'--'+FULL_FLAG_ARGNAME,
                        action='store_true',
                        help='Output/Input uses the full store. ',
                        default=use_full_value)
    g_full_notfull.add_argument('-'+NOT_FULL_FLAG_ARGNAME_ALIAS,'--'+NOT_FULL_FLAG_ARGNAME,
                        action='store_true',
                        help='Output/Input uses NOT the full store. ',
                        default=not use_full_value)
    return parser


def add_output_argument(parser: argparse.ArgumentParser=None)->argparse.ArgumentParser:
    """
    Adds an output argument to the given argument parser.

    Args:
        parser (argparse.ArgumentParser): The argument parser to add the output argument to.

    Returns:
        parser (argparse.ArgumentParser): The argument parser with the output argument added.
    """
    global default_parser
    if parser is None:
        parser=default_parser
        
    parser.add_argument('-o','--output',
                        action='store_true',
                        help='Output PATH. If specified, output will be written in the filestore.')
    
    return parser

def add_compressed_argument(parser: argparse.ArgumentParser=None)->argparse.ArgumentParser:
    """
    Adds an compressed argument to the given argument parser.
    
    Args:
        parser (argparse.ArgumentParser): The argument parser to add the output argument to.
        
    Returns:
        parser (argparse.ArgumentParser): The argument parser with argument added.
    """
    global default_parser
    if parser is None:
        parser=default_parser
    group_output=_get_group_by_title(parser,ARG_GROUP_OUTPUT_TITLE,ARG_GROUP_OUTPUT_DESCRIPTION)
    group_output.add_argument('-z','--compress',
                        action='store_true',
                        help='Compress the output. If specified, it will also activate the output flag.')
    return parser


def add_use_full_argument(parser: argparse.ArgumentParser=None,load_from_settings=True)->argparse.ArgumentParser:
    """
    Adds a use full argument to the given argument parser.
    
    Args:
        parser (argparse.ArgumentParser): The argument parser to add the read full argument to.
        
    Returns:
        parser (argparse.ArgumentParser): The argument parser with argument added.
    """
    global default_parser
    if parser is None:
        parser=default_parser
    
    #print("DEPRECATION: Use: add_bars_amount_V2_arguments")
    full_notfull_group = parser.add_mutually_exclusive_group()
    use_full_value=load_arg_default_from_settings(FULL_FLAG_ARGNAME,False,FULL_FLAG_ARGNAME_ALIAS) if load_from_settings else False
    full_notfull_group.add_argument('-'+FULL_FLAG_ARGNAME_ALIAS,'--'+FULL_FLAG_ARGNAME,
                        action='store_true',
                        help='Output/Input uses the full store. ',
                        default=use_full_value)
    full_notfull_group.add_argument('-'+NOT_FULL_FLAG_ARGNAME_ALIAS,'--'+NOT_FULL_FLAG_ARGNAME,
                        action='store_true',
                        help='Output/Input uses NOT the full store. ',
                        default=not use_full_value)
 
    return parser

def add_use_fresh_argument(parser: argparse.ArgumentParser=None,load_from_settings=True)->argparse.ArgumentParser:
    """
    Adds a use fresh argument to the given argument parser.
    
    Args:
        parser (argparse.ArgumentParser): The argument parser to add the use fresh argument to.
        
    Returns:
        parser (argparse.ArgumentParser): The argument parser with argument added.
    """
    global default_parser
    if parser is None:
        parser=default_parser
    bars_group=_get_group_by_title(parser,ARG_GROUP_BARS_TITLE,ARG_GROUP_BARS_DESCRIPTION)
    
    fresh_old_group=bars_group.add_mutually_exclusive_group()
    use_fresh_value=load_arg_default_from_settings(FRESH_FLAG_ARGNAME,False,FRESH_FLAG_ARGNAME_ALIAS) if load_from_settings else False
    fresh_old_group.add_argument('-'+FRESH_FLAG_ARGNAME_ALIAS,'--'+FRESH_FLAG_ARGNAME,
                        action='store_true',
                        help='Freshening the storage with latest market. ',
                        default=use_fresh_value)
    fresh_old_group.add_argument('-'+NOT_FRESH_FLAG_ARGNAME_ALIAS,'--'+NOT_FRESH_FLAG_ARGNAME,
                        action='store_true',
                        help='Output/Input wont be freshed from storage (weekend or tests). ',
                        default=not use_fresh_value)
 
    return parser


def add_keepbidask_argument(parser: argparse.ArgumentParser=None,load_default_from_settings=True,flag_default_value=True)->argparse.ArgumentParser:
    """
    Adds a keep Bid/Ask argument to the given argument parser.
    
    Args:
        parser (argparse.ArgumentParser): The argument parser to add the keep bid/ask argument to.
        
    Returns:
        parser (argparse.ArgumentParser): The argument parser with the argument added.
    """
    global default_parser
    if parser is None:
        parser=default_parser
    
    cleanupGroup=_get_group_by_title(parser,ARG_GROUP_CLEANUP_TITLE,ARG_GROUP_CLEANUP_DESCRIPTION)
    group_kba=cleanupGroup.add_mutually_exclusive_group()
    
    default_value = load_arg_default_from_settings(KEEP_BID_ASK_FLAG_ARGNAME,flag_default_value,alias=KEEP_BID_ASK_FLAG_ARGNAME_ALIAS) if load_default_from_settings else flag_default_value
    #print("keepbidask  value:"+str(default_value))
    group_kba.add_argument('-'+KEEP_BID_ASK_FLAG_ARGNAME_ALIAS,'--'+KEEP_BID_ASK_FLAG_ARGNAME,
                        action='store_true',
                        help='Keep Bid/Ask in storage. ',
                        default=default_value)
    group_kba.add_argument('-'+REMOVE_BID_ASK_FLAG_ARGNAME_ALIAS,'--'+REMOVE_BID_ASK_FLAG_ARGNAME,
                        action='store_true',
                        help='Remove Bid/Ask in storage. ',
                        default=not default_value)
    return parser

def add_format_outputs_arguments(parser:argparse.ArgumentParser=None,load_from_settings=True)->argparse.ArgumentParser:
  global default_parser
  if parser is None:
    parser=default_parser
  
  out_group=_get_group_by_title(parser,"Outputs")
  json_flag_default_value=load_arg_default_from_settings("json_output",False,"json") if load_from_settings else False
  f_exclusive=out_group.add_mutually_exclusive_group()
  f_exclusive.add_argument("-json", "--json_output", help="Output in JSON format", action="store_true",default=json_flag_default_value)
  #Markdown
  markdown_flag_default_value=load_arg_default_from_settings("markdown_output",False,"md") if load_from_settings else False
  f_exclusive.add_argument("-md", "--markdown_output", help="Output in Markdown format", action="store_true",default=markdown_flag_default_value)
  return parser

def add_patterns_arguments(parser:argparse.ArgumentParser=None,load_from_settings=True)->argparse.ArgumentParser:
  global default_parser
  if parser is None:
    parser=default_parser
  
  pn_group=_get_group_by_title(parser,"Patterns")
  clh_default_value=load_arg_default_from_settings("columns_list_from_higher_tf",None,"clh") if load_from_settings else None
  pn_group.add_argument("-clh", "--columns_list_from_higher_tf", nargs='+', help="List of columns to get from higher TF.  Default is mfi_sig,zone_sig,ao", default=clh_default_value)
  
  pn_default_value=load_arg_default_from_settings("patternname",None,"pn") if load_from_settings else None
  pn_group.add_argument("-pn", "--patternname", help="Pattern Name",default=pn_default_value)
  
  #patterns_list_value=load_arg_default_from_settings("patterns",None) if load_from_settings else None
  pn_group.add_argument("-pls", "--list-patterns", help="List Patterns", action="store_true")
  
  #Add the format outputs
  parser=add_format_outputs_arguments(parser,load_from_settings)
  return parser













import jgtclirqdata


def add_jgtclirqdata_arguments(parser: argparse.ArgumentParser=None)->argparse.ArgumentParser:
    global default_parser
    if parser is None:
        parser=default_parser
    group_pattern=parser.add_argument_group('RQ Pattern', 'RQ Pattern to use.  Future practice to create request patterns to load into the args later.')
    
    group_pattern.add_argument('-pdsrq','--pds_rq_base',
                        action='store_true',
                        help='Use PDS_RQ JSON_BASE')
    #PDS_RQ JSON_NORMAL
    group_pattern.add_argument('-pdsrqn','--pds_rq_normal',
                        action='store_true',
                        help='Use PDS_RQ JSON_NORMAL')
    #PDS_RQ JSON_NORMAL_FRESH
    group_pattern.add_argument('-pdsrqnf','--pds_rq_normal_fresh',
                        action='store_true',
                        help='Use PDS_RQ JSON_NORMAL_FRESH')
    #PDS_RQ JSON_FULL
    group_pattern.add_argument('-pdsrqf','--pds_rq_full',
                        action='store_true',
                        help='Use PDS_RQ JSON_FULL')
    #PDS_RQ JSON_FULL_FRESH
    group_pattern.add_argument('-pdsrqff','--pds_rq_full_fresh',
                        action='store_true',
                        help='Use PDS_RQ JSON_FULL_FRESH')
    #IDS_RQ JSON_BASE
    group_pattern.add_argument('-idsrq','--ids_rq_base',
                        action='store_true',
                        help='Use IDS_RQ JSON_BASE')
    
    group_pattern.add_argument('-cdsrq','--cds_rq_normal',
                        action='store_true',
                        help='Use CDS_RQ JSON_NORMAL')
    
    group_pattern.add_argument('-cdsrqf','--cds_rq_full',
                        action='store_true',
                        help='Use CDS_RQ JSON_FULL')
    #CDS_RQ JSON_FULL_FRESH
    group_pattern.add_argument('-cdsrqff','--cds_rq_full_fresh',
                        action='store_true',
                        help='Use CDS_RQ JSON_FULL_FRESH')
    #CDS_RQ JSON_NORM_FRESH
    group_pattern.add_argument('-cdsrqnf','--cds_rq_norm_fresh',
                        action='store_true',
                        help='Use CDS_RQ JSON_NORM_FRESH')
    
    return parser

#post add_jgtclirqdata_arguments
def __jgtclirqdata_post_parse():
    global args
    __check_if_parsed()
    _jgtclirqdata_to_load=[jgtclirqdata.PDS_RQ_BASE]
    try:
        if hasattr(args, 'pds_rq_base') and args.pds_rq_base:
            _jgtclirqdata_to_load.append(jgtclirqdata.PDS_RQ_BASE)
        if hasattr(args, 'ids_rq_base') and args.ids_rq_base:
            _jgtclirqdata_to_load.append(jgtclirqdata.IDS_RQ_BASE)
        if hasattr(args, 'cds_rq_normal') and args.cds_rq_normal:
            _jgtclirqdata_to_load.append(jgtclirqdata.CDS_RQ_NORMAL)
        if hasattr(args, 'cds_rq_full') and args.cds_rq_full:
            _jgtclirqdata_to_load.append(jgtclirqdata.CDS_RQ_FULL)
        if hasattr(args, 'cds_rq_full_fresh') and args.cds_rq_full_fresh:
            _jgtclirqdata_to_load.append(jgtclirqdata.CDS_RQ_FULL_FRESH)
        if hasattr(args, 'cds_rq_norm_fresh') and args.cds_rq_norm_fresh:
            _jgtclirqdata_to_load.append(jgtclirqdata.CDS_RQ_NORM_FRESH)
        if hasattr(args, 'pds_rq_normal') and args.pds_rq_normal:
            _jgtclirqdata_to_load.append(jgtclirqdata.PDS_RQ_NORMAL)
        if hasattr(args, 'pds_rq_normal_fresh') and args.pds_rq_normal_fresh:
            _jgtclirqdata_to_load.append(jgtclirqdata.PDS_RQ_NORMAL_FRESH)
        if hasattr(args, 'pds_rq_full') and args.pds_rq_full:
            _jgtclirqdata_to_load.append(jgtclirqdata.PDS_RQ_FULL)
        if hasattr(args, 'pds_rq_full_fresh') and args.pds_rq_full_fresh:
            _jgtclirqdata_to_load.append(jgtclirqdata.PDS_RQ_FULL_FRESH)
            
    except:
        pass
    #for each pattern we have, load their key/value into the args
    for pattern in _jgtclirqdata_to_load:
        try:
            #print(pattern)
            json_obj = json.loads(pattern)
            for key in json_obj:
                setattr(args, key, json_obj[key])
        except:
            pass
    return args


#Load a json content from the argument --json
def add_load_json_file_argument(parser: argparse.ArgumentParser=None)->argparse.ArgumentParser:
    global default_parser
    if parser is None:
        parser=default_parser
    output_group=_get_group_by_title(parser,ARG_GROUP_OUTPUT_TITLE,ARG_GROUP_OUTPUT_DESCRIPTION)
    output_group.add_argument('-jsonf','--json_file',
                        help='JSON filepath content to be loaded.')
    
    return parser


def __json_post_parse():
    global args
    __check_if_parsed()
    
    try:
        #Create args from the json_file
        if hasattr(args, 'json_file') and args.json_file is not None:
            filepath = args.json_file
            #raise exception if file does not exist
            if not os.path.exists(filepath):
                raise Exception("File does not exist."+filepath)
            with open(filepath, 'r') as f:
                try:
                    json_obj = json.load(f)
                    for key in json_obj:
                        #print("key:"+key, " value:"+str(json_obj[key]))
                        setattr(args, key, json_obj[key])
                except:
                    pass
    except:
        pass
    return args


def add_exit_if_error(parser: argparse.ArgumentParser=None)->argparse.ArgumentParser:
    global default_parser
    if parser is None:
        parser=default_parser
        
    parser.add_argument('-xe','--exitonerror',
                        action='store_true',
                        help='Exit on error rather than trying to keep looking')
    return parser

def add_dropna_volume_argument(parser: argparse.ArgumentParser=None)->argparse.ArgumentParser:
    global default_parser
    if parser is None:
        parser=default_parser
    
    cleanupGroup=_get_group_by_title(parser,ARG_GROUP_CLEANUP_TITLE,ARG_GROUP_CLEANUP_DESCRIPTION)
    
    dv_group=cleanupGroup.add_mutually_exclusive_group()
    
    dv_group.add_argument('-'+DROPNA_VOLUME_FLAG_ARGNAME_ALIAS,'--'+DROPNA_VOLUME_FLAG_ARGNAME,
                        action='store_true',
                        help='Drop rows with NaN (or 0) in volume column.  (note.Montly chart does not dropna volume)')
    
    dv_group.add_argument("-"+DONT_DROPNA_VOLUME_FLAG_ARGNAME_ALIAS,"--"+DONT_DROPNA_VOLUME_FLAG_ARGNAME, help="Do not dropna volume", action="store_true")
    
    return parser



def __dropna_volume__post_parse():
    try:
        dropna_volume_flag = _do_we_dropna_volume(args)
        setattr(args, 'dropna_volume',dropna_volume_flag)
        #dont_dropna_volume
        setattr(args, 'dont_dropna_volume',not dropna_volume_flag)
        
    except:
        pass
    return args

def __quiet__post_parse():
    try:
        if not hasattr(args, 'quiet') and (hasattr(args, 'verbose') and args.verbose==0):
            #add quiet to list
           #print("Quiet mode activated in parser")
           setattr(args, 'quiet', True)
        else:
            setattr(args, 'quiet', False)
    except:
        pass
    return args



def __timeframes_post_parse()->argparse.Namespace:
    global args,settings
    __check_if_parsed()
    
    _timeframes=None
    
    
    if hasattr(args, "timeframes"):
        _timeframes=getattr(args, "timeframes")
    else:
        try:
            if settings["timeframes"]:
                _timeframes =settings["timeframes"]
        except:
            pass
            
            
    if not isinstance(_timeframes, list) and _timeframes is not None:
        _timeframes=parse_timeframes_helper(_timeframes)

    if _timeframes is None and hasattr(args, 'timeframe'):
        _timeframes=getattr(args, 'timeframe')
        #if we have coma in the string
        if _timeframes is not None and "," in _timeframes:
            _timeframes=parse_timeframes_helper(_timeframes)
        else:
            _timeframes = os.getenv("T",None)
            
    setattr(args, 'timeframes',_timeframes)

    return args

from jgtconstants import TIMEFRAMES_ALL, TIMEFRAMES_DEFAULT


def parse_timeframes_helper(timeframes):
    if timeframes in TIMEFRAMES_ALL:
        return [timeframes]
    
    if timeframes == "default":
        __timeframes = TIMEFRAMES_DEFAULT
    if  timeframes == "all" :
        __timeframes = TIMEFRAMES_ALL
    
    if timeframes == "default" or timeframes == "all" :
        try:
            _timeframes = os.getenv("T",__timeframes).split(",")
        except:
            _timeframes = None
    else:
        try:
            _timeframes = timeframes.split(",")
        except:
            _timeframes = None
    return _timeframes

def __crop_last_dt__post_parse()->argparse.Namespace:
    global args
    __check_if_parsed()
    if hasattr(args, 'crop_last_dt'):
        if args.crop_last_dt is not None:
            setattr(args, 'crop_last_dt', args.crop_last_dt)
    else:
        setattr(args, 'crop_last_dt', None)
    return args

#@STCIssue We want this to Default to True and would be flagged to false by rm_bid_ask
def __keep_bid_ask__post_parse(keep_bid_ask_argname = 'keepbidask',rm_bid_ask_argname = 'rmbidask')->argparse.Namespace:
    global args
    __check_if_parsed()
    try:
        keep_bid_ask_value=True        
        
        if hasattr(args, rm_bid_ask_argname) or hasattr(args,'rm_bid_ask'):
            if hasattr(args, rm_bid_ask_argname) and args.rmbidask:
                keep_bid_ask_value=False
            if hasattr(args, 'rm_bid_ask') and args.rm_bid_ask:
                keep_bid_ask_value=False
        
        setattr(args, keep_bid_ask_argname,keep_bid_ask_value)
        setattr(args, 'keep_bid_ask',keep_bid_ask_value) # Future refactoring will be called just that.
        setattr(args, rm_bid_ask_argname,not keep_bid_ask_value)
    except:
        pass
    return args
    
def __verbose__post_parse()->argparse.Namespace:
    global args
    __check_if_parsed()
    try:
        if not hasattr(args, 'verbose'):
            setattr(args, 'verbose',0)
    except:
        pass
    return args

def __quotescount__post_parse()->argparse.Namespace:
    global args
    __check_if_parsed()
    try:
        if not hasattr(args, QUOTES_COUNT_ARGNAME):
            setattr(args, QUOTES_COUNT_ARGNAME,-1)
    except:
        pass
    return args

def __balligator_flag__post_parse()->argparse.Namespace:
    global args
    __check_if_parsed()
    try:
        if not hasattr(args, BALLIGATOR_FLAG_ARGNAME):
            setattr(args, BALLIGATOR_FLAG_ARGNAME,False)
            
        if hasattr(args, BALLIGATOR_FLAG_ARGNAME) and args.timeframe=="M1":
            #print("We dont do balligator for M1")
            setattr(args, BALLIGATOR_FLAG_ARGNAME,False)
    except:
        pass
    return args

def __talligator_flag__post_parse()->argparse.Namespace:
    global args
    __check_if_parsed()
    try:
        if not hasattr(args, TALLIGATOR_FLAG_ARGNAME):
            setattr(args, TALLIGATOR_FLAG_ARGNAME,False)
            
        if hasattr(args, TALLIGATOR_FLAG_ARGNAME) and args.timeframe=="M1":
            #print("We dont do talligator for M1")
            setattr(args, TALLIGATOR_FLAG_ARGNAME,False)
        
        if hasattr(args, TALLIGATOR_FLAG_ARGNAME) and args.timeframe=="W1":
            #print("We dont do talligator for W1")
            setattr(args, TALLIGATOR_FLAG_ARGNAME,False)
    except:
        pass
    return args


_NO_MFI_FOR_M1_flag=False
def __mfi_flag__post_parse()->argparse.Namespace:
    global args
    __check_if_parsed()
    try:
        if not hasattr(args, MFI_FLAG_ARGNAME):
            setattr(args, MFI_FLAG_ARGNAME,False)
        
        if _NO_MFI_FOR_M1_flag:
            if hasattr(args, MFI_FLAG_ARGNAME) and args.timeframe=="M1":
                #print("We dont do MFI for M1")
                setattr(args, MFI_FLAG_ARGNAME,False)
    except:
        pass
    return args


def __use_fresh__post_parse()->argparse.Namespace: 
    global args
    __check_if_parsed()
    try:
        if not hasattr(args, FRESH_FLAG_ARGNAME):
            setattr(args, FRESH_FLAG_ARGNAME,False)
        if hasattr(args, FRESH_FLAG_ARGNAME) and args.fresh:
            setattr(args, FRESH_FLAG_ARGNAME,True)
        else:
            if hasattr(args, NOT_FRESH_FLAG_ARGNAME) and args.notfresh:
                setattr(args, FRESH_FLAG_ARGNAME,False)
    except:
        pass
    return args

def __check_if_parsed():
    if args is None or args==[]:
        raise Exception("args is not set.  Run parse_args() first before calling this function.  Most likely, the CLI must be updated to do parser.parse_args() first instead of doing it in the main (REFACTORING Responsabilities)")

def _post_parse_dependent_arguments_rules()->argparse.Namespace:
    global args
    __check_if_parsed()
    
    #args=_load_settings_from_args() #@STCIssue - THis has to load before we do any other argparsing, like preloading the settings.
    
    args=__quiet__post_parse()
    
    
    # OTHER DEPENDENT RULES

    args=__dropna_volume__post_parse()
    
    args=__keep_bid_ask__post_parse()
    
    args=__timeframes_post_parse()
    args=__crop_last_dt__post_parse()
    args=__verbose__post_parse()
    args=__quotescount__post_parse()
    args=__balligator_flag__post_parse()
    args=__talligator_flag__post_parse()
    args=__mfi_flag__post_parse()
    args=__use_fresh__post_parse()
    args=__json_post_parse()   
    args=__jgtclirqdata_post_parse()
    args=_demo_flag()
    return args

    
def _demo_flag():
    global args
    if hasattr(args, 'demo') and args.demo:
        setattr(args, 'connection', 'Demo')
        setattr(args, 'demo', True)
    else:
        setattr(args, 'connection', 'Real')
        setattr(args, 'demo', False)
    return args

def parse_args(parser: argparse.ArgumentParser=None)->argparse.Namespace:
    global default_parser,args,settings
    if parser is None:
        parser=default_parser
    args= parser.parse_args()
    try:
        #set a key jgtcommon_settings in the args to store settings
        setattr(args, 'jgtcommon_settings', get_settings())
    except:
        pass
    
    
    args=_post_parse_dependent_arguments_rules()
    return args

def _do_we_dropna_volume(_args=None):
    global args
    if _args is None:
        _args=args
    dropna_volume_value = _args.dropna_volume or not _args.dont_dropna_volume
    if args.timeframe == "M1" and dropna_volume_value:
        print("We dont dropna volume for M1")
        return False # We dont drop for Monthly
    return dropna_volume_value

def add_viewpath_argument(parser: argparse.ArgumentParser=None)->argparse.ArgumentParser:
    """
    Adds an view path argument to the given argument parser.
    
    Args:
        parser (argparse.ArgumentParser): The argument parser to add the viewpath argument to.
        
    Returns:
        parser (argparse.ArgumentParser): The argument parser with the argument added.
    """
    global default_parser
    if parser is None:
        parser=default_parser
    output_group=_get_group_by_title(parser,ARG_GROUP_OUTPUT_TITLE,ARG_GROUP_OUTPUT_DESCRIPTION)
    output_group.add_argument('-vp','--viewpath',
                        action='store_true',
                        dest='viewpath',
                        help='flag to just view the path of files from arguments -i -t.')
    return parser


# def add_quiet_argument(parser: argparse.ArgumentParser=None)->argparse.ArgumentParser:
#     parser.add_argument('-q','--quiet',
#                         action='store_true',
#                         help='Suppress all output. If specified, no output will be printed to the console.')
#     return parser

def add_verbose_argument(parser: argparse.ArgumentParser=None)->argparse.ArgumentParser:

    global default_parser
    if parser is None:
        parser=default_parser
    
    group_verbosity=_get_group_by_title(parser,ARG_GROUP_VERBOSITY_TITLE,ARG_GROUP_VERBOSITY_DESCRIPTION)
    
    group_verbosity.add_argument('-v', '--verbose',
                        type=int,
                        default=0,
                        help='Set the verbosity level. 0 = quiet, 1 = normal, 2 = verbose, 3 = very verbose, etc.')
    return parser

def add_cds_argument(parser: argparse.ArgumentParser=None)->argparse.ArgumentParser:

    global default_parser
    if parser is None:
        parser=default_parser

    parser.add_argument('-cds','--cds',
                        action='store_true',
                        default=False,
                        help='Action the creation of CDS')
    return parser

def add_ids_argument(parser: argparse.ArgumentParser=None)->argparse.ArgumentParser:

    global default_parser
    if parser is None:
        parser=default_parser

    parser.add_argument('-ids','--ids',
                        action='store_true',
                        default=False,
                        help='Action the creation of IDS')
    return parser

def add_ids_mfi_argument(parser: argparse.ArgumentParser=None,load_default_from_settings=True,flag_default_value=True)->argparse.ArgumentParser:

    global default_parser
    if parser is None:
        parser=default_parser
    default_value = load_arg_default_from_settings(MFI_FLAG_ARGNAME,flag_default_value,alias=MFI_FLAG_ARGNAME_ALIAS) if load_default_from_settings else flag_default_value
    #settings.get(MFI_FLAG_ARGNAME,flag_default_value) if settings else flag_default_value
    
    group_indicators=_get_group_by_title(parser,ARG_GROUP_INDICATOR_TITLE,ARG_GROUP_INDICATOR_DESCRIPTION)
    mfi_exclusive_subgroup=group_indicators.add_mutually_exclusive_group()
    mfi_exclusive_subgroup.add_argument(
        "-"+MFI_FLAG_ARGNAME_ALIAS,
        "--"+MFI_FLAG_ARGNAME,
        action="store_true",
        default=default_value,
        help="Enable the Market Facilitation Index indicator.",
    )
    mfi_exclusive_subgroup.add_argument(
        "-"+NO_MFI_FLAG_ARGNAME_ALIAS,
        "--"+NO_MFI_FLAG_ARGNAME,  
        action="store_true",
        default=not default_value,
        help="Disable the Market Facilitation Index indicator.",
    )  
    return parser

def add_ids_gator_oscillator_argument(parser: argparse.ArgumentParser=None,load_default_from_settings=True,flag_default_value=False)->argparse.ArgumentParser:

    global default_parser
    if parser is None:
        parser=default_parser

    group_indicators=_get_group_by_title(parser,ARG_GROUP_INDICATOR_TITLE,ARG_GROUP_INDICATOR_DESCRIPTION)
    
    default_value = load_arg_default_from_settings(GATOR_OSCILLATOR_FLAG_ARGNAME,flag_default_value,alias=GATOR_OSCILLATOR_FLAG_ARGNAME_ALIAS) if load_default_from_settings else flag_default_value
    
    group_indicators.add_argument(
        "-"+GATOR_OSCILLATOR_FLAG_ARGNAME_ALIAS,
        "--"+GATOR_OSCILLATOR_FLAG_ARGNAME,
        action="store_true",
        help="Enable the Gator Oscillator indicator.",
        default=default_value
    )
    return parser

from jgtcliconstants import  (LARGEST_FRACTAL_PERIOD_ARGNAME,
                                LARGEST_FRACTAL_PERIOD_ARGNAME_ALIAS, SETTING_ARGNAME, SETTING_ARGNAME_ALIAS,)
def add_ids_fractal_largest_period_argument(parser: argparse.ArgumentParser=None,load_default_from_settings=True,default_value=89)->argparse.ArgumentParser:

    global default_parser
    if parser is None:
        parser=default_parser
    group_indicators=_get_group_by_title(parser,ARG_GROUP_INDICATOR_TITLE,ARG_GROUP_INDICATOR_DESCRIPTION)
    
    default_value = load_arg_default_from_settings(LARGEST_FRACTAL_PERIOD_ARGNAME,default_value,alias=LARGEST_FRACTAL_PERIOD_ARGNAME_ALIAS) if load_default_from_settings else default_value
    
    group_indicators.add_argument(
        "-"+LARGEST_FRACTAL_PERIOD_ARGNAME_ALIAS,
        "--"+LARGEST_FRACTAL_PERIOD_ARGNAME,
        type=int,
        default=default_value,
        help=f"The largest fractal period. ({default_value})",
    )
    return parser



from jgtconstants import BJAW_PERIODS,BTEETH_PERIODS,BLIPS_PERIODS
from jgtcliconstants import  (BALLIGATOR_PERIOD_JAWS_ARGNAME,
                              BALLIGATOR_PERIOD_JAWS_ARGNAME_ALIAS,
                              BALLIGATOR_PERIOD_TEETH_ARGNAME,
                              BALLIGATOR_PERIOD_TEETH_ARGNAME_ALIAS,
                              BALLIGATOR_PERIOD_LIPS_ARGNAME,
                              BALLIGATOR_PERIOD_LIPS_ARGNAME_ALIAS,)
def add_ids_balligator_argument(parser: argparse.ArgumentParser=None,add_periods_arg=True,load_default_from_settings=True,flag_default_value=False,period_jaws_default = BJAW_PERIODS,period_teeth_default=BTEETH_PERIODS,period_lips_default=BLIPS_PERIODS)->argparse.ArgumentParser:

    global default_parser
    if parser is None:
        parser=default_parser
    
    flag_default_value=load_arg_default_from_settings(BALLIGATOR_FLAG_ARGNAME,flag_default_value,alias=BALLIGATOR_FLAG_ARGNAME_ALIAS) if load_default_from_settings else flag_default_value
    
    group_indicators=_get_group_by_title(parser,ARG_GROUP_INDICATOR_TITLE,ARG_GROUP_INDICATOR_DESCRIPTION)
    group_indicators.add_argument(
        "-"+BALLIGATOR_FLAG_ARGNAME_ALIAS,
        "--"+BALLIGATOR_FLAG_ARGNAME,
        action="store_true",
        help="Enable the Big Alligator indicator.",
        default=flag_default_value
    )
    
    if add_periods_arg:
        balligator_period_jaws_default=load_arg_default_from_settings(BALLIGATOR_PERIOD_JAWS_ARGNAME,period_jaws_default,alias=BALLIGATOR_PERIOD_JAWS_ARGNAME_ALIAS)
        
        group_indicators.add_argument(
            "-"+BALLIGATOR_PERIOD_JAWS_ARGNAME_ALIAS,
            "--"+BALLIGATOR_PERIOD_JAWS_ARGNAME,
            type=int,
            default=balligator_period_jaws_default,
            help="The period of the Big Alligator jaws.",
        )
        balligator_period_teeth_default=load_arg_default_from_settings(BALLIGATOR_PERIOD_TEETH_ARGNAME,period_teeth_default,alias=BALLIGATOR_PERIOD_TEETH_ARGNAME_ALIAS)
        
        group_indicators.add_argument(
            "-"+BALLIGATOR_PERIOD_TEETH_ARGNAME_ALIAS,
            "--"+BALLIGATOR_PERIOD_TEETH_ARGNAME,
            type=int,
            default=balligator_period_teeth_default,
            help="The period of the Big Alligator teeth.",
        )
        balligator_period_lips_default=load_arg_default_from_settings(BALLIGATOR_PERIOD_LIPS_ARGNAME,period_lips_default,alias=BALLIGATOR_PERIOD_LIPS_ARGNAME_ALIAS)
        
        group_indicators.add_argument(
            "-"+BALLIGATOR_PERIOD_LIPS_ARGNAME_ALIAS,
            "--"+BALLIGATOR_PERIOD_LIPS_ARGNAME,
            type=int,
            default=balligator_period_lips_default,
            help="The period of the Big Alligator lips.",
        )
    return parser

from jgtconstants import TJAW_PERIODS,TTEETH_PERIODS,TLIPS_PERIODS
from jgtcliconstants import  (TALLIGATOR_PERIOD_JAWS_ARGNAME,
                              TALLIGATOR_PERIOD_JAWS_ARGNAME_ALIAS,
                              TALLIGATOR_PERIOD_TEETH_ARGNAME,
                              TALLIGATOR_PERIOD_TEETH_ARGNAME_ALIAS,
                              TALLIGATOR_PERIOD_LIPS_ARGNAME,
                              TALLIGATOR_PERIOD_LIPS_ARGNAME_ALIAS,)
def add_ids_talligator_argument(parser: argparse.ArgumentParser=None,add_periods_arg=True,load_default_from_settings=True,flag_default_value=False,period_jaws_default = TJAW_PERIODS,period_teeth_default=TTEETH_PERIODS,period_lips_default=TLIPS_PERIODS)->argparse.ArgumentParser:

    global default_parser
    if parser is None:
        parser=default_parser
    
    flag_default_value=load_arg_default_from_settings(TALLIGATOR_FLAG_ARGNAME,flag_default_value,alias=TALLIGATOR_FLAG_ARGNAME_ALIAS) if load_default_from_settings else flag_default_value
    
    group_indicators=_get_group_by_title(parser,ARG_GROUP_INDICATOR_TITLE,ARG_GROUP_INDICATOR_DESCRIPTION)
    group_indicators.add_argument(
        "-"+TALLIGATOR_FLAG_ARGNAME_ALIAS,
        "--"+TALLIGATOR_FLAG_ARGNAME,
        action="store_true",
        help="Enable the Tide Alligator indicator.",
        default=flag_default_value
    )
    
    if add_periods_arg:
        talligator_period_jaws_default=load_arg_default_from_settings(TALLIGATOR_PERIOD_JAWS_ARGNAME,period_jaws_default,alias=TALLIGATOR_PERIOD_JAWS_ARGNAME_ALIAS)
        
        group_indicators.add_argument(
            "-"+TALLIGATOR_PERIOD_JAWS_ARGNAME_ALIAS,
            "--"+TALLIGATOR_PERIOD_JAWS_ARGNAME,
            type=int,
            default=talligator_period_jaws_default,
            help="The period of the Tide Alligator jaws.",
        )
        talligator_period_teeth_default=load_arg_default_from_settings(TALLIGATOR_PERIOD_TEETH_ARGNAME,period_teeth_default,alias=TALLIGATOR_PERIOD_TEETH_ARGNAME_ALIAS)
        
        group_indicators.add_argument(
            "-"+TALLIGATOR_PERIOD_TEETH_ARGNAME_ALIAS,
            "--"+TALLIGATOR_PERIOD_TEETH_ARGNAME,
            type=int,
            default=talligator_period_teeth_default,
            help="The period of the Tide Alligator teeth.",
        )
        talligator_period_lips_default=load_arg_default_from_settings(TALLIGATOR_PERIOD_LIPS_ARGNAME,period_lips_default,alias=TALLIGATOR_PERIOD_LIPS_ARGNAME_ALIAS)
        
        group_indicators.add_argument(
            "-"+TALLIGATOR_PERIOD_LIPS_ARGNAME_ALIAS,
            "--"+TALLIGATOR_PERIOD_LIPS_ARGNAME,
            type=int,
            default=talligator_period_lips_default,
            help="The period of the Tide Alligator lips.",
        )
    return parser

def add_ads_argument(parser: argparse.ArgumentParser=None)->argparse.ArgumentParser:

    global default_parser
    if parser is None:
        parser=default_parser
    interaction_group=_get_group_by_title(parser,ARG_GROUP_INTERACTION_TITLE,ARG_GROUP_INTERACTION_DESCRIPTION)
    interaction_group.add_argument('-ads','--ads',
                        action='store_true',
                        default=False,
                        help='Action the creation of ADS and show the chart')
    return parser

def add_iprop_init_argument(parser: argparse.ArgumentParser=None)->argparse.ArgumentParser:

    global default_parser
    if parser is None:
        parser=default_parser

    parser.add_argument('-iprop','--iprop',
                        action='store_true',
                        default=False,
                        help='Toggle the downloads of all instrument properties ')
    return parser

def add_debug_argument(parser: argparse.ArgumentParser=None)->argparse.ArgumentParser:

    global default_parser
    if parser is None:
        parser=default_parser

    group_verbosity=_get_group_by_title(parser,ARG_GROUP_VERBOSITY_TITLE,ARG_GROUP_VERBOSITY_DESCRIPTION)
    group_verbosity.add_argument('-debug','--debug',
                        action='store_true',
                        default=False,
                        help='Toggle debug ')
    return parser

def add_pdsserver_argument(parser: argparse.ArgumentParser=None)->argparse.ArgumentParser:

    global default_parser
    if parser is None:
        parser=default_parser

    parser.add_argument('-server','--server',
                        action='store_true',
                        default=False,
                        help='Run the server ')
    return parser


def print_exception(exception: Exception):
    #logging.error("Exception: {0}\n{1}".format(exception, traceback.format_exc()))
    print("Exception: {0}\n{1}".format(exception, traceback.format_exc()))






def diff_month(year: int, month: int, date2: datetime):
    return (year - date2.year) * 12 + month - date2.month



def export_env_if_any(config):
    # if has a key : "keep_bid_ask" and if yes and set to "true", export an env variable "JGT_KEEP_BID_ASK" to "1"
    if 'keep_bid_ask' in config and config['keep_bid_ask'] == True:
        os.environ['JGT_KEEP_BID_ASK'] = '1'

_JGT_CONFIG_JSON_SECRET=None

def readconfig(json_config_str=None,config_file = 'config.json',export_env=False,config_file_path_env_name='JGT_CONFIG_PATH',config_values_env_name='JGT_CONFIG',force_read_json=False,demo=False,use_demo_json_config=False):
    global _JGT_CONFIG_JSON_SECRET
    
    try:
        home_dir = os.path.expanduser("~")
    except:
        home_dir=os.environ["HOME"]
    if home_dir=="":
        home_dir=os.environ["HOME"]
        
    #demo_config are assumed to be $HOME/.jgt/config_demo.json
    if demo and use_demo_json_config:
        config_file = os.path.join(home_dir, '.jgt/config_demo.json')
        #check if exist, advise and raise exception if not
        if not os.path.exists(config_file):
            print("Configuration not found. create : {config_file} or we will try to use the _demo in the usual config.json")
            config=readconfig(force_read_json=True)
            _set_demo_credential(config,demo)
            return config
            #raise Exception(f"Configuration not found. create : {config_file})")
 
    #force_read_json are assumed to be $HOME/.jgt/config.json
    if force_read_json:
        config_file = os.path.join(home_dir, '.jgt/config.json')
        #check if exist, advise and raise exception if not
        if not os.path.exists(config_file):
            raise Exception(f"Configuration not found. create : {config_file})")
        #load and return config
        with open(config_file, 'r') as f:
            config = json.load(f)
            if export_env:
                export_env_if_any(config)
            _set_demo_credential(config,demo)
            return config
            
    
    #print argument values to debug
    _DEBUG_240619=False
    if _DEBUG_240619:
        print("json_config_str:",json_config_str)
        print("config_file:",config_file)
        print("export_env:",export_env)
        print("config_file_path_env_name:",config_file_path_env_name)
        print("config_values_env_name:",config_values_env_name)
    
    # Try reading config file from current directory

    if json_config_str is not None:
        config = json.loads(json_config_str)
        _JGT_CONFIG_JSON_SECRET=json_config_str
        if export_env:
            export_env_if_any(config)
        _set_demo_credential(config,demo)
        return config
    
    
    if _JGT_CONFIG_JSON_SECRET is not None:
        config = json.loads(_JGT_CONFIG_JSON_SECRET)
        if export_env:
            export_env_if_any(config)
        _set_demo_credential(config,demo)
        return config
    
    config = None

    # if file does not exist try set the path to the file in the HOME
    if not os.path.exists(config_file):
        config_file = os.path.join(home_dir, config_file)
        
    if os.path.exists(config_file):
        with open(config_file, 'r') as f:
            config = json.load(f)
            if export_env:
                export_env_if_any(config)
            _set_demo_credential(config,demo)
            return config
    else:

        config_file = os.path.join(home_dir, config_file)
        if os.path.isfile(config_file):
            with open(config_file, 'r') as f:
                config = json.load(f)
        else:
            # If config file still not found, try reading from environment variable
            config_json_str = os.getenv('JGT_CONFIG_JSON_SECRET')
            if config_json_str:
                config = json.loads(config_json_str)
                if export_env:
                    export_env_if_any(config)
                _set_demo_credential(config,demo)
                return config


    # Now you can use the config dictionary in your application

    # if file dont exist, try loading from env var JGT_CONFIG
    if not os.path.exists(config_file):
        config_json_str = os.getenv(config_values_env_name)
        
        if config_json_str:
            config = json.loads(config_json_str)
            if export_env:
                export_env_if_any(config)
            #return config
        else:
            # if not found, try loading from env var JGT_CONFIG_PATH
            config_file = os.getenv(config_file_path_env_name)
            if config_file:
                with open(config_file, 'r') as f:
                    config = json.load(f)
                    if export_env:
                        export_env_if_any(config)
                    #return config
           # else:
               
    # Read config file
    if config is None:
        #print("config_file:",config_file)
        if config_file is not None and os.path.exists(config_file) :
            with open(config_file, 'r') as file:
                config = json.load(file)
        
    if config is None:
        #Last attempt to read
        try:
            another_config = "config.json"
            if not os.path.exists(another_config):
                another_config = os.path.join(os.path.expanduser('~'), '.jgt', 'config.json')
                
            if not os.path.exists(another_config):
                another_config = "/etc/jgt/config.json"
            with open(another_config, 'r') as file:
                config = json.load(file)
        except:
            pass
        # try:
        #     home_config_path_try2 = os.path.join(os.path.expanduser('~'), '.jgt', 'config.json')
        #     if not os.path.exists(another_config):
        #         another_config = "/etc/jgt/config.json"
        #     with open(another_config, 'r') as file:
        #         config = json.load(file)
        # except:
        #     pass
                
        if config is None:    
            raise Exception(f"Configuration not found. Please provide a config file or set the JGT_CONFIG environment variable to the JSON config string. (config_file={config_file})")
    
    if export_env:
        export_env_if_any(config)
    _set_demo_credential(config,demo)
    return config

def _set_demo_credential(config,demo=False):
    if demo:
        config["user_id"]=config["user_id_demo"]
        config["password"]=config["password_demo"]
        config["account"]=config["account_demo"]
        config["connection"]="Demo"


def read_fx_str_from_config(demo=False)->tuple[str,str,str,str,str]:
    config = readconfig(demo=demo)
    if config["connection"]=="Real" and demo: #Make sure we have our demo credentials
        _set_demo_credential(config,True)
    str_user_id=config['user_id']
    str_password=config['password']
    str_url=config['url']
    str_connection="Real" if not demo else "Demo"
    str_account=config['account']
    return str_user_id,str_password,str_url,str_connection,str_account



def is_market_open(current_time=None):
    if current_time is None:
        current_time = datetime.utcnow()

    # Define market open and close times
    market_open_time = time(21, 0)  # 21:00 UTC
    market_close_time = time(21, 15)  # 21:15 UTC

    # Get the current day of the week (0=Monday, 6=Sunday)
    current_day = current_time.weekday()

    current_time_utc = current_time.time()
    # Check if the market is open
    if current_day == 6:  # Sunday
        if current_time_utc >= market_open_time:
            return True
    elif current_day == 4:  # Friday
        if current_time_utc < market_close_time:
            return True
    elif 0 <= current_day < 4:  # Monday to Thursday
        return True

    return False


import unittest
from datetime import datetime, timezone

from jgtcommon import is_market_open


class TestMarketOpen(unittest.TestCase):
    def test_market_open_sunday(self):
        # Simulate Sunday at 21:00 UTC
        test_time = datetime(2023, 10, 1, 21, 0)  # Example date that is a Sunday
        self.assertTrue(is_market_open(test_time))

    def test_market_open_monday(self):
        # Simulate Monday at 10:00 UTC
        test_time = datetime(2023, 10, 2, 10, 0)  # Example date that is a Monday
        self.assertTrue(is_market_open(test_time))

    def test_market_open_friday_before_close(self):
        # Simulate Friday at 20:00 UTC
        test_time = datetime(2023, 10, 6, 20, 0)  # Example date that is a Friday
        self.assertTrue(is_market_open(test_time))
    
    def test_market_open_friday_just_before_close(self):
        # Simulate Friday at 21:00 UTC
        test_time = datetime(2024, 8, 9, 21, 14,0,0,tzinfo=timezone.utc)  # Example date that is a Friday when it is just closed
        result = is_market_open(test_time)
        self.assertTrue(result)

class TestMarketClosed(unittest.TestCase):
    def test_market_closed_saturday(self):
        # Simulate Saturday at 10:00 UTC
        test_time = datetime(2023, 10, 7, 10, 0)  # Example date that is a Saturday
        result = is_market_open(test_time)
        self.assertFalse(result)

    def test_market_closed_sunday(self):
        # Simulate Sunday at 10:00 UTC
        test_time = datetime(2023, 10, 8, 10, 0)  # Example date that is a Sunday
        result = is_market_open(test_time)
        self.assertFalse(result)

    def test_market_closed_friday_after_close(self):
        # Simulate Friday at 21:00 UTC
        test_time = datetime(2024, 8, 9, 21, 16,0,0,tzinfo=timezone.utc)  # Example date that is a Friday when it is just closed
        result = is_market_open(test_time)
        self.assertFalse(result)



    
if __name__ == '__main__':
    unittest.main()